from collections.abc import Iterable, Sequence
import itertools
import os
from pathlib import Path
import shutil
from unittest import mock

from PySide6 import QtCore, QtWidgets
from PySide6.QtTest import QTest
from PySide6.QtWidgets import QPushButton, QTreeWidgetItemIterator, QWidget

from asammdf.gui.widgets.batch import BatchWidget
from test.asammdf.gui.test_base import TestBase


class TestBatchWidget(TestBase):
    testResult = None
    concatenate_aspect = 0
    modify_aspect = 1
    stack_aspect = 2
    bus_aspect = 3
    default_test_file = "ASAP2_Demo_V171.mf4"

    def setUp(self):
        super().setUp()
        self.widget = None
        self.plot = None

        patcher = mock.patch("asammdf.gui.widgets.file.ErrorDialog")
        self.mc_widget_ed = patcher.start()
        self.addCleanup(patcher.stop)

        patcher_settings = mock.patch("asammdf.gui.widgets.batch.QtCore.QSettings", spec=QtCore.QSettings)
        self.mc_settings = patcher_settings.start()
        self.mc_settings.return_value.value.side_effect = itertools.chain(["Natural sort"], itertools.repeat(None))
        self.mc_settings.return_value.setValue.return_value = None
        self.addCleanup(patcher_settings.stop)

        patcher_restore = mock.patch.object(BatchWidget, "restore_export_settings")
        self.mc_restore = patcher_restore.start()
        self.addCleanup(patcher_restore.stop)

        self.processEvents()

    def setUpBatchWidget(self, *args, measurement_files: Sequence[str] | None):
        """
        Created because a lot of testcases,
        we do not need other parameters for BatchWidget initialization.
        """
        if measurement_files is None:
            self.widget = BatchWidget(*args)
            self.processEvents()

        else:
            self.widget = BatchWidget(*args)
            self.processEvents()
            for file in measurement_files:
                self.assertTrue(Path(file).exists())
            self.widget.files_list.addItems(measurement_files)

            # Evaluate that all files was opened
            self.assertEqual(self.widget.files_list.count(), len(measurement_files))

        self.widget.showNormal()

    def copy_mdf_files_to_workspace(self):
        # copy mf4 files from resources to test workspace
        for file in os.listdir(self.resource):
            if file.endswith((".mf4", ".mdf")):
                shutil.copyfile(Path(self.resource, file), Path(self.test_workspace, file))

    def select_channels(self, channels_list: Iterable[str | int]) -> list:
        """
        Select channels from a list of names or indexes.

        Parameters
        ----------
        channels_list: a list of channel names or indexes

        Returns
        -------
        channels names list
        """
        self.selected_channels = []
        iterator = QTreeWidgetItemIterator(self.widget.filter_tree)

        count = 0
        while iterator.value():
            item = iterator.value()
            if (
                (item.name in channels_list or count in channels_list)
                and "time" not in item.name.lower()
                and "$" not in item.name
            ):  # by name or index, exclde time and calibration channels
                item.setCheckState(0, QtCore.Qt.CheckState.Checked)
                self.assertTrue(item.checkState(0) == QtCore.Qt.CheckState.Checked)
                self.selected_channels.append(item.name)
                self.processEvents()

            iterator += 1
            count += 1

        # Evaluate that channels were added to "selected_filter_channels"
        for index in range(self.widget.selected_filter_channels.count()):
            item = self.widget.selected_filter_channels.item(index)
            self.assertIn(item.text(), self.selected_channels)

        return self.selected_channels

    def get_selected_groups(self, channels: list) -> dict:
        self.widget.filter_view.setCurrentText("Internal file structure")
        self.processEvents(1)

        groups = {}
        iterator = QTreeWidgetItemIterator(self.widget.filter_tree)
        count = 0

        while iterator.value():
            item = iterator.value()
            if item.text(0) in channels and item.text(0) != "time":
                if item.parent().text(0) not in groups.keys():
                    groups[item.parent().text(0)] = [item.text(0)]
                else:
                    groups[item.parent().text(0)].append(item.text(0))

            iterator += 1
            count += 1
        return groups

    def mouse_click_on_btn_with_progress(self, btn: QPushButton):
        """
        Click on the button with progress bar and wait until progress bar will be closed.

        Parameters
        ----------
        btn := QPushButton
        """
        # Mouse click on button
        QTest.mouseClick(btn, QtCore.Qt.MouseButton.LeftButton)
        # Wait for progress bar thread to finish
        while self.widget._progress:
            self.processEvents(0.01)
        self.processEvents()

    def toggle_checkboxes(self, widget: QWidget, check=True):
        """
        Iterate given widget to find QCheckBox items.
        All founded checkboxes will have 'check' check state

        Parameters
        ----------
        widget := widget to iterate
        check := check state (True/False)

        """
        # set checkboxes check state
        for checkbox in widget.findChildren(QtWidgets.QCheckBox):
            if checkbox.isChecked() != check:
                self.mouseClick_CheckboxButton(checkbox)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'windows_selection_dialog.ui'
##
## Created by: Qt User Interface Compiler version 6.7.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractButton, QApplication, QCheckBox, QDialog,
    QDialogButtonBox, QFrame, QGroupBox, QScrollArea,
    QSizePolicy, QVBoxLayout, QWidget)

class Ui_WindowSelectionDialog(object):
    def setupUi(self, WindowSelectionDialog):
        if not WindowSelectionDialog.objectName():
            WindowSelectionDialog.setObjectName(u"WindowSelectionDialog")
        WindowSelectionDialog.resize(453, 435)
        self.verticalLayout_2 = QVBoxLayout(WindowSelectionDialog)
        self.verticalLayout_2.setSpacing(1)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(1, 1, 1, 1)
        self.groupBox = QGroupBox(WindowSelectionDialog)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setFlat(True)
        self.lay = QVBoxLayout(self.groupBox)
        self.lay.setSpacing(1)
        self.lay.setObjectName(u"lay")
        self.lay.setContentsMargins(1, 1, 1, 1)
        self.scrollArea = QScrollArea(self.groupBox)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setFrameShape(QFrame.Shape.NoFrame)
        self.scrollArea.setFrameShadow(QFrame.Shadow.Plain)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents_2 = QWidget()
        self.scrollAreaWidgetContents_2.setObjectName(u"scrollAreaWidgetContents_2")
        self.scrollAreaWidgetContents_2.setGeometry(QRect(0, 0, 439, 355))
        self.verticalLayout_3 = QVBoxLayout(self.scrollAreaWidgetContents_2)
        self.verticalLayout_3.setSpacing(1)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(1, 1, 1, 1)
        self.selection_layout = QVBoxLayout()
        self.selection_layout.setSpacing(1)
        self.selection_layout.setObjectName(u"selection_layout")

        self.verticalLayout_3.addLayout(self.selection_layout)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents_2)

        self.lay.addWidget(self.scrollArea)

        self.disable_channels = QCheckBox(self.groupBox)
        self.disable_channels.setObjectName(u"disable_channels")

        self.lay.addWidget(self.disable_channels)


        self.verticalLayout_2.addWidget(self.groupBox)

        self.buttonBox = QDialogButtonBox(WindowSelectionDialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Cancel|QDialogButtonBox.StandardButton.Ok)

        self.verticalLayout_2.addWidget(self.buttonBox)


        self.retranslateUi(WindowSelectionDialog)
        self.buttonBox.accepted.connect(WindowSelectionDialog.accept)
        self.buttonBox.rejected.connect(WindowSelectionDialog.reject)

        QMetaObject.connectSlotsByName(WindowSelectionDialog)
    # setupUi

    def retranslateUi(self, WindowSelectionDialog):
        WindowSelectionDialog.setWindowTitle(QCoreApplication.translate("WindowSelectionDialog", u"Select window type", None))
        self.groupBox.setTitle(QCoreApplication.translate("WindowSelectionDialog", u"Available window types", None))
        self.disable_channels.setText(QCoreApplication.translate("WindowSelectionDialog", u"Disable newly added channels", None))
    # retranslateUi



import json
import os
import pathlib
from random import randint
import shutil
from unittest import mock

from PySide6 import QtCore, QtTest, QtWidgets

from asammdf.gui.widgets.file import FileWidget
from asammdf.gui.widgets.numeric import Numeric
from asammdf.gui.widgets.plot import Plot
from test.asammdf.gui.test_base import TestBase


class TestFileWidget(TestBase):
    testResult = None

    def setUp(self):
        super().setUp()
        self.widget = None
        self.plot = None

        patcher = mock.patch("asammdf.gui.widgets.file.ErrorDialog")
        self.mc_widget_ed = patcher.start()
        self.addCleanup(patcher.stop)

        self.measurement_file = shutil.copy(pathlib.Path(self.resource, "ASAP2_Demo_V171.mf4"), self.test_workspace)

    def tearDown(self):
        if self.widget is not None:
            self.widget.close()
        super().tearDown()

    def setUpFileWidget(self, *args, measurement_file, default):
        """
        Created because for a lot of testcases,
        we do not need other parameters for FileWidget initialization.
        """
        if default:
            self.widget = FileWidget(
                measurement_file,
                True,  # with_dots
                True,  # subplots
                True,  # subplots_link
                False,  # ignore_value2text_conversions
                False,  # display_cg_name
                "line",  # line_interconnect
                "",  # password
                False,  # hide_missing_channels
                False,  # hide_disabled_channels
            )
        else:
            self.widget = FileWidget(measurement_file, *args)
        self.processEvents()
        self.widget.showMaximized()

    def create_window(self, window_type, channels_names=(), channels_indexes=()):
        channel_tree = self.widget.channels_tree
        channel_tree.clearSelection()
        for channel in channels_names:
            channel = self.find_channel(channel_tree, channel_name=channel)
            channel.setCheckState(0, QtCore.Qt.CheckState.Checked)
        for channel in channels_indexes:
            channel = self.find_channel(channel_tree, channel_index=channel)
            channel.setCheckState(0, QtCore.Qt.CheckState.Checked)

        with mock.patch("asammdf.gui.widgets.file.WindowSelectionDialog") as mc_WindowSelectionDialog:
            mc_WindowSelectionDialog.return_value.result.return_value = True
            mc_WindowSelectionDialog.return_value.selected_type.return_value = window_type
            # - Press PushButton "Create Window"
            QtTest.QTest.mouseClick(self.widget.create_window_btn, QtCore.Qt.MouseButton.LeftButton)
            widget_types = self.get_sub_windows()
            self.assertIn(window_type, widget_types)

    @staticmethod
    def find_channel(channel_tree, channel_name=None, channel_index=None):
        selected_channel = None
        if not channel_name and not channel_index:
            selected_channel = channel_tree.topLevelItem(0)
        elif channel_index:
            selected_channel = channel_tree.topLevelItem(channel_index)
        elif channel_name:
            iterator = QtWidgets.QTreeWidgetItemIterator(channel_tree)
            while iterator.value():
                item = iterator.value()
                if item and item.text(0) == channel_name:
                    selected_channel = item
                    break
                iterator += 1
        return selected_channel

    def add_channels(self, channels_list: list, widget=None):
        """
        Add channels to the widget from a list using channels indexes or channels names
        Add channels to the list <self.channels>

        Parameters
            channel_list: a list with existent channels names and indexes;
            widget: the widget where the channels will be inserted.

        Returns
            None: if one channel or widget does not exist;
            self.channels: if all channels were found.
        """
        if channels_list is None:
            channels_list = [randint(5, self.widget.channels_tree.topLevelItemCount() - 10)]
        if not isinstance(channels_list, list):
            return []
        # Ensure "Natural sort" mode for channel view
        self.widget.channel_view.setCurrentIndex(0)
        self.processEvents(0.1)
        windows_list = self.widget.mdi_area.subWindowList()
        if len(windows_list) == 0:
            return []
        else:
            if widget is None:
                widget = windows_list[0].widget()
            else:
                if widget not in [w.widget() for w in windows_list]:
                    return []

        channels = []
        for channel in channels_list:
            found_channel = None
            if isinstance(channel, int):
                found_channel = self.find_channel(channel_tree=self.widget.channels_tree, channel_index=channel)
            elif isinstance(channel, str):
                found_channel = self.find_channel(channel_tree=self.widget.channels_tree, channel_name=channel)
            self.processEvents(0.1)
            if found_channel is not None:
                channels.append(found_channel)
        self.assertEqual(
            len(channels_list),
            len(channels),
            msg=f"Not all channels from given list was found!      \n"
            f"Given channels:\n{channels_list} \nFounded channels: \n"
            f"\n{channels}\n++++++++++++++++++++\nwidget:\t\t{widget}",
        )

        # add channels to channel selection
        self.widget.add_new_channels([channel.name for channel in channels], widget)

        if isinstance(widget, Numeric):
            cw = widget.channels.dataView
            self.channels = cw.backend.signals
        elif isinstance(widget, Plot):
            cw = widget.channel_selection
            self.channels = [cw.topLevelItem(_) for _ in range(cw.topLevelItemCount())]
        else:
            return []
        self.processEvents(0.01)
        return self.channels

    def load_shortcuts_from_json_file(self, widget):
        """
        Used to store widget shortcuts into variable "self.shortcuts"

        Parameters
        ----------
            widget: tested widget

        Returns
        -------
            None: if class not exist in json
            Dict: if class was found; dict contains all shortcuts applied to selected class
        """
        # get a json path
        json_path = os.path.normpath(os.path.join(os.path.dirname(__file__), "Shortcuts", "Shortcuts.json"))
        widget_name = widget.__class__.__name__
        self.shortcuts = None
        with open(json_path) as json_file:
            data = json.load(json_file)
            if widget_name in data.keys():
                self.shortcuts = data[widget_name][0]
        json_file.close()
        return self.shortcuts

    def get_sub_windows(self):
        widget_types = sorted(w.widget().__class__.__name__ for w in self.widget.mdi_area.subWindowList())
        return widget_types

    def load_display_file(self, display_file):
        with (
            mock.patch.object(self.widget, "load_window", wraps=self.widget.load_window) as mo_load_window,
            mock.patch("asammdf.gui.widgets.file.QtWidgets.QFileDialog.getOpenFileName") as mo_getOpenFileName,
        ):
            mo_getOpenFileName.return_value = display_file, None
            QtTest.QTest.mouseClick(
                self.widget.load_channel_list_btn,
                QtCore.Qt.MouseButton.LeftButton,
            )
            # Pre-Evaluate
            mo_load_window.assert_called()



#!/usr/bin/env python\

import pathlib
from unittest import mock

from PySide6.QtCore import QPoint
from PySide6.QtGui import QColor, QKeySequence, Qt
from PySide6.QtTest import QTest
from PySide6.QtWidgets import QTreeWidgetItemIterator

from test.asammdf.gui.test_base import OpenMDF, Pixmap
from test.asammdf.gui.widgets.test_BaseFileWidget import TestFileWidget
from test.asammdf.gui.widgets.test_BasePlotWidget import TestPlotWidget


class TestTreeWidgetShortcuts(TestFileWidget):
    def setUp(self):
        """
        Events:
         - Open measurement file

        Returns
        -------

        """
        super().setUp()
        # Open measurement file
        measurement_file = str(pathlib.Path(self.resource, "ASAP2_Demo_V171.mf4"))

        # Open measurement file
        self.setUpFileWidget(measurement_file=measurement_file, default=True)
        self.tw = self.widget.channels_tree  # TreeWidget

        # get shortcuts
        self.assertIsNotNone(self.load_shortcuts_from_json_file(self.tw))

    def test_toggle_select_channel_shortcut(self):
        """
        Test scope:
            Check if by pressing key Space, check status of tree item is changed

        Events:
            - Select all even channels and press key Space
            - Select the same channels and press again key Space

        Evaluate:
            - Evaluate that check state of all channels is "Unchecked" at start
            - Evaluate that check state of even channels is "Checked" after
                even channels became selected and key Space is pressed
            - Evaluate that check state of all channels is "Unchecked" after
                even channels became selected second time and key Space is pressed
        Returns
        -------

        """
        items_count = self.tw.topLevelItemCount()
        for _ in range(items_count - 1):
            # Evaluate that all items is ...
            self.assertEqual(self.tw.topLevelItem(_).checkState(0), Qt.CheckState.Unchecked)

            # Select all impar items and hit the Space key
            if _ % 2 == 1:
                self.tw.topLevelItem(_).setSelected(True)
                QTest.keySequence(self.tw, QKeySequence(self.shortcuts["toggle_check_state"]))
                self.tw.topLevelItem(_).setSelected(False)

        self.processEvents()

        # Evaluate that all impar items are checked
        for _ in range(items_count - 1):
            if _ % 2 == 1:
                self.assertEqual(self.tw.topLevelItem(_).checkState(0), Qt.CheckState.Checked)

                # Select item and hit again the Space key
                self.tw.topLevelItem(_).setSelected(True)
                QTest.keySequence(self.tw, QKeySequence(self.shortcuts["toggle_check_state"]))
                self.tw.topLevelItem(_).setSelected(False)
            else:
                self.assertEqual(self.tw.topLevelItem(_).checkState(0), Qt.CheckState.Unchecked)

        # Evaluate that all items are unchecked
        for _ in range(items_count - 1):
            self.assertEqual(self.tw.topLevelItem(_).checkState(0), Qt.CheckState.Unchecked)


class TestChannelsTreeWidgetShortcuts(TestPlotWidget):
    """
    Tests for Ctrl+C, Ctrl+V was tested in test_PlotWidget_Shortcuts
    """

    def setUp(self):
        """
        Events:
            - Open measurement file
            - Create a plot window
        Returns
        -------

        """
        super().setUp()
        # Open measurement file
        measurement_file = str(pathlib.Path(self.resource, "ASAP2_Demo_V171.mf4"))
        # Open measurement file
        self.setUpFileWidget(measurement_file=measurement_file, default=True)
        # Select channels -> Press PushButton "Create Window" -> "Plot"
        self.create_window(window_type="Plot")

        self.ctw = self.widget.mdi_area.subWindowList()[0].widget().channel_selection  # ChannelsTreeWidget

        # get shortcuts
        self.assertIsNotNone(self.load_shortcuts_from_json_file(self.ctw))

    def test_delete_shortcut(self):
        """
        Test Scope:
            - Ensure that key Delete will remove selected channels.

        Events:
            - Add some channels to plot.
            - Select first item and press key Delete.
            - Select all items and press key delete.

        Evaluate:
            - Evaluate that there are all items added to plot.
            - Evaluate that first item is removed from a list after it became selected and key Delete is pressed.
            - Evaluate that all items after they became selected are removed by pressing key delete.
        Returns
        -------

        """

        self.assertIsNotNone(self.add_channels([10, 11, 12, 13]))
        channel_count = self.ctw.topLevelItemCount()
        # Click on last channel
        channel_0 = self.channels.pop()
        self.mouseClick_WidgetItem(channel_0)
        # Press key Delete
        QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["delete_items"]))

        # Evaluate
        self.assertEqual(self.ctw.topLevelItemCount(), channel_count - 1)
        iterator = QTreeWidgetItemIterator(self.ctw)
        while item := iterator.value():
            self.assertNotEqual(channel_0, item)
            iterator += 1

        # select all items
        QTest.keySequence(self.ctw, QKeySequence("Ctrl+A"))
        # Press key Delete
        QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["delete_items"]))

        # Evaluate
        self.assertEqual(self.ctw.topLevelItemCount(), 0)

    def test_add_pattern_based_channel_group_shortcut(self):
        """
        Test Scope:
            Check if a new pattern-based channel group can be created by pressing shortcut "Ctrl+Insert"

        Events:
            - Press "Ctrl+Insert"
            - Simulate dialog window fill with:
                    "pattern": f"*{Filter}*",
                    "match_type": "Wildcard",
                    "case_sensitive": False,
                    "filter_type": "Unspecified",
                    "filter_value": 0.0,
                    "raw": False,
                    "ranges": [],
                    "name": "Matrix",
                    "integer_format": "phys".

        Evaluate:
            - Evaluate that method exec_() of AdvancedSearch object was called.
            - Evaluate that in plot channel selection exists a new group with specific name and pattern dict.
            - Evaluate that group contains all channels from measurement with a specific sequence of string
                in their name.
        """
        group_name = "Matrix"
        result = {
            "pattern": f"*{group_name}*",
            "match_type": "Wildcard",
            "case_sensitive": False,
            "filter_type": "Unspecified",
            "filter_value": 0.0,
            "raw": False,
            "ranges": [],
            "name": f"{group_name}",
            "integer_format": "phys",
        }
        # mock for QInputDialog object
        with mock.patch("asammdf.gui.widgets.tree.AdvancedSearch") as mo_AdvancedSearch:
            mo_AdvancedSearch.return_value.result = result
            # Press Ctrl+Insert
            QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["add_pattern_based_group"]))
        # Evaluate
        mo_AdvancedSearch.return_value.exec_.assert_called()
        self.assertEqual(1, self.ctw.topLevelItemCount())
        group = self.ctw.topLevelItem(0)
        self.assertEqual(group.name, result["name"])
        self.assertDictEqual(group.pattern, result)
        # Store all channels names from a group in a list
        group_channels_name = [channel.name for channel in group.get_all_channel_items()]
        # Store all channels with specific pattern in their names in a list
        items = []
        with OpenMDF(self.measurement_file) as mdf:
            items.extend(ch.name for ch in mdf.iter_channels() if group_name.upper() in ch.name.upper())
        self.assertEqual(len(group_channels_name), len(items))
        for channel_name in group_channels_name:
            self.assertIn(channel_name, items)
            # To avoid duplicates
            items.remove(channel_name)

    def test_add_channel_group(self):
        """
        Test Scope:
            Check if a new channel group can be created by pressing shortcut "Shift+Insert"
        Events:
            - Press "Shift+Insert"
            - Type some text and press "Ok"
        Evaluate:
            - Evaluate that method getText() of QInputDialog object was called
            - Evaluate that in plot channel selection exist new group with specific name
        """
        # mock for QInputDialog object
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QInputDialog") as mo_QInputDialog:
            mo_QInputDialog.getText.return_value = (self.id(), True)
            # Press Shift+Insert
            QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["add_channel_group"]))
        # Evaluate
        mo_QInputDialog.getText.assert_called()
        self.assertEqual(self.ctw.topLevelItem(0).name, self.id())

    def test_select_channel_shortcut(self):
        """
        Test scope:
            Ensure that key Space change checks state for selected item.
        Events:
            - Add few channels to plot
            - Click on first channel -> press key Space.
            - Click on rest channels -> press key Space.
            - Click in first channel -> press key Space.

        Evaluate
            - Evaluate that all items are checked by default.
            - Evaluate that by selecting one item from a list and pressing key space, it states became "Unchecked".
            - Evaluate that by selecting all checked items and pressing key space, its states became "Unchecked".
            - Evaluate that by selecting one unchecked item and pressing key space, it states became "Checked".
        Returns
        -------

        """
        self.assertIsNotNone(self.add_channels([10, 11, 12, 13]))

        iterator = QTreeWidgetItemIterator(self.ctw)
        while item := iterator.value():
            self.assertEqual(item.checkState(0), Qt.CheckState.Checked)
            iterator += 1
        # Select click on first item -> press key space
        self.mouseClick_WidgetItem(self.ctw.topLevelItem(0))
        QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["toggle_check_state"]))

        self.assertEqual(self.ctw.topLevelItem(0).checkState(0), Qt.CheckState.Unchecked)

        for _ in range(1, self.ctw.topLevelItemCount()):
            self.mouseClick_WidgetItem(self.ctw.topLevelItem(_))
            QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["toggle_check_state"]))

        iterator = QTreeWidgetItemIterator(self.ctw)
        while item := iterator.value():
            self.assertEqual(item.checkState(0), Qt.CheckState.Unchecked)
            iterator += 1

        # Select click on first item -> press key space
        self.mouseClick_WidgetItem(self.ctw.topLevelItem(0))
        QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["toggle_check_state"]))
        self.assertEqual(self.ctw.topLevelItem(0).checkState(0), Qt.CheckState.Checked)

    def test_color_shortcut(self):
        """
        Test Scope:
            - Ensure that channel color is changed.
        Events:
            - Add 2 channels to Plot.
            - Press C
            - Select 1 Channel -> Press C
            - Select 2 Channels -> Press C
        Evaluate:
            - Evaluate that there is successful added channels to plot.
            - Evaluate that color dialog is not open if channel is not selected.
            - Evaluate that channel color is changed only for a selected channel.
        """
        self.assertIsNotNone(self.add_channels([10, 11, 12]))
        with self.subTest("test_WOSelectedChannel"):
            with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QColorDialog.getColor") as mo_getColor:
                QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["set_color"]))
                mo_getColor.assert_not_called()

        with self.subTest("test_1SelectedChannel"):
            with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QColorDialog.getColor") as mo_getColor:
                # Setup
                self.mouseClick_WidgetItem(self.channels[0])
                previous_color = self.channels[0].color.name()
                color = QColor("magenta")
                mo_getColor.return_value = color
                # Event
                QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["set_color"]))
                # Evaluate
                mo_getColor.assert_called()
                self.assertNotEqual(previous_color, self.channels[0].color.name())
                self.assertEqual(color.name(), self.channels[0].color.name())

        with self.subTest("test_allSelectedChannel"):
            with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QColorDialog.getColor") as mo_getColor:
                # Setup
                # Set selected both channels
                QTest.keySequence(self.ctw, QKeySequence("Ctrl+A"))
                # store previous colors of channels
                previous_ch_colors = [channel.color.name() for channel in self.channels]
                color = QColor("cyan") if "#00FFFF" not in previous_ch_colors else QColor("black")
                mo_getColor.return_value = color
                # Event
                QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["set_color"]))
                # Evaluate
                mo_getColor.assert_called()
                for channel in self.channels:
                    self.assertNotIn(channel.color.name(), previous_ch_colors)
                    self.assertEqual(channel.color.name(), color.name())

    def test_copy_display_properties_shortcut(self):
        """
        Test Scope:
            - Ensure that key "Ctrl+Shift+C" can copy display properties of selected item

        Events:
            - Open Plot with one channel
            - Select this channel
            - Press Ctrl+Shift+C

        Evaluate:
            - Evaluate that there is added one item to plot
            - Evaluate that in clipboard is display properties of selected item
        """
        self.assertIsNotNone(self.add_channels([10]))
        # Evaluate precondition
        ch_0_display_properties = self.ctw.topLevelItem(0).get_display_properties()
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QApplication.instance") as mo_instance:
            # Click on a first channel
            self.mouseClick_WidgetItem(self.channels[0])
            # Press Ctrl+Shift+C
            QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["copy_display_properties"]))
        # Evaluate
        mo_instance.return_value.clipboard.return_value.setText.assert_called_with(ch_0_display_properties)

    def test_paste_display_properties_shortcut(self):
        """
        Test Scope:
            - Ensure that key "Ctrl+Shift+V" can paste from clipboard display properties to selected item.

        Events:
            - Open Plot with two channels.
            - Copy display properties of the first item.
            - Select second item -> press Ctrl+Shift+C.

        Evaluate:
            - Evaluate that to plot are added two items.
            - Evaluate that display properties of channels are not identical.
            - Evaluate that after selecting second channel and hit keys "Ctrl+Shift+V",
                the second channel has the same display properties with the first channel.
        """
        self.assertIsNotNone(self.add_channels([10, 11]))
        # Evaluate precondition
        ch_0_display_properties = self.ctw.topLevelItem(0).get_display_properties()
        self.assertNotEqual(ch_0_display_properties, self.ctw.topLevelItem(1).get_display_properties())
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QApplication.instance") as mo_instance:
            mo_instance.return_value.clipboard.return_value.text.return_value = ch_0_display_properties

            # Click on a first channel
            self.mouseClick_WidgetItem(self.channels[1])
            # Press Ctrl+Shift+C
            QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["paste_display_properties"]))
        # Evaluate
        mo_instance.return_value.clipboard.return_value.text.assert_called()
        self.assertEqual(
            self.ctw.topLevelItem(0).get_display_properties(), self.ctw.topLevelItem(1).get_display_properties()
        )

    def test_copy_names_shortcut(self):
        """
        Test Scope:
            - Ensure that key "Ctrl+Shift+V" can paste from clipboard display properties to selected item.

        Events:
            - Open Plot with few channels.
            - Select first item -> press Ctrl+N.
            - Select all items -> press Ctrl+N.

        Evaluate:
            - Evaluate that to plot are added few items.
            - Evaluate that after selecting one channel and pressing key Ctrl+N, in clipboard is added it name.
            - Evaluate that after selecting all channels and pressing key Ctrl+N, in clipboard is added items names.
        Returns
        -------

        """
        self.assertIsNotNone(self.add_channels([10, 11, 12, 13]))
        expected_cb_all_items_call = ""
        iterator = QTreeWidgetItemIterator(self.ctw)
        while item := iterator.value():
            expected_cb_all_items_call += "\n" + item.name
            iterator += 1
        expected_cb_all_items_call = expected_cb_all_items_call.split("\n", 1)[1]

        channel_0 = self.ctw.topLevelItem(0)
        with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QApplication.instance") as mo_instance:
            # Select click on first item -> press key Ctrl+N
            self.mouseClick_WidgetItem(channel_0)
            QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["copy_names"]))
            # Evaluate
            mo_instance.return_value.clipboard.return_value.setText.assert_called_with(self.ctw.topLevelItem(0).name)

            # Select all items
            QTest.keySequence(self.ctw, QKeySequence("Ctrl+A"))
            #  Press key Ctrl+N
            QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["copy_names"]))
            # Evaluate
            mo_instance.return_value.clipboard.return_value.setText.assert_called_with(expected_cb_all_items_call)

    def test_set_color_range_shortcut(self):
        """
        Test Scope:
            Check if color range is triggered after pressing key Ctrl+R.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Display 1 signal on plot.
            - Select signal.
            - Press "Ctrl+R" -> set ranges from 0 to 40% of y value and colors green and red -> apply.
            - Click on unchanged color part of signal on plot.
            - Click on changed color part of signal on plot.
            - Click Ctrl+G to shift plot from and to 40% of y range.
        Evaluate:
            - Evaluate that plot is not black.
            - Evaluate that plot selected channel value has channel color.
            - Evaluate RangeEditor object was called.
            - Evaluate that plot selected channel value area doesn't have red and green colors,
                only the original one, when the cursor does not intersect the red part of the signal.
            - Evaluate that after clicking on part that enter in selected ranges,
                plot selected channel value area has only red and green colors.
            - Evaluate using Y axis scaling if plot is correctly painted:
                from 0 to 40% it is red, from 40% is not affected.
        """
        # Setup
        # add channels to plot
        self.assertIsNotNone(self.add_channels([35]))
        plot = self.widget.mdi_area.subWindowList()[0].widget()
        self.processEvents(0.01)
        if plot.selected_channel_value_btn.isFlat():
            QTest.mouseClick(plot.selected_channel_value_btn, Qt.MouseButton.LeftButton)
        if not plot.bookmark_btn.isFlat():
            QTest.mouseClick(plot.bookmark_btn, Qt.MouseButton.LeftButton)
        self.mouseClick_WidgetItem(self.channels[0])

        plot.plot.set_dots(False)

        # Evaluate
        self.assertTrue(Pixmap.has_color(plot.selected_channel_value.grab(), self.channels[0].color.name()))
        y_range = plot.plot.y_axis.range[1] - plot.plot.y_axis.range[0]
        offset = 40
        red_range = y_range * offset / 100
        green = QColor.fromRgbF(0.000000, 1.000000, 0.000000, 1.000000)
        red = QColor.fromRgbF(1.000000, 0.000000, 0.000000, 1.000000)

        range_editor_result = [
            {
                "background_color": green,
                "font_color": red,
                "op1": "<=",
                "op2": "<=",
                "value1": 0.0,
                "value2": red_range,
            }
        ]

        # Click on channel
        self.mouseClick_WidgetItem(self.channels[0])
        # Sometimes, because small display resolution, mouse click on qItem isn't performed properly on linux
        if not plot.channel_selection.topLevelItem(0).isSelected():
            plot.channel_selection.topLevelItem(0).setSelected(True)
        self.processEvents(0.01)

        with mock.patch("asammdf.gui.widgets.tree.RangeEditor") as mo_RangeEditor:
            mo_RangeEditor.return_value.result = range_editor_result
            mo_RangeEditor.return_value.pressed_button = "apply"
            # Press "Alt+R"
            QTest.keySequence(self.ctw, QKeySequence(self.shortcuts["set_color_range"]))

        # Evaluate
        mo_RangeEditor.assert_called()
        self.assertEqual(self.channels[0].ranges, range_editor_result)

        self.mouseClick_WidgetItem(self.channels[0])
        for _ in range(100):
            self.processEvents(None)

        # Displayed signal
        sig = self.channels[0].signal
        # Coordinates for viewbox, second dot in the center of visual
        min_ = med_ = max_ = None
        for _ in range(len(sig)):
            if sig.samples[_] == 0 and (sig.samples[_ + 10] - sig.samples[_ - 10]) < (sig.samples.max() / 10):
                min_ = _
                break
        for _ in range(min_, len(sig)):
            if sig.samples[_] == red_range:
                med_ = _
                break
        for _ in range(med_, len(sig)):
            if sig.samples[_] == sig.samples.max():
                max_ = _
                break
        x, y, w, h = sig.timestamps[min_], sig.samples[min_], sig.timestamps[med_], sig.samples[med_]
        # Set X and Y ranges for viewbox
        plot.plot.viewbox.setXRange(x, w, padding=0)
        plot.plot.viewbox.setYRange(y, h, padding=0)
        self.processEvents(None)

        # Click in the middle of the plot
        QTest.mouseClick(
            plot.plot.viewport(),
            Qt.MouseButton.LeftButton,
            Qt.KeyboardModifier.NoModifier,
            QPoint(int(plot.plot.width() / 2), int(plot.plot.height() / 2)),
        )
        self.processEvents(0.1)
        for _ in range(100):
            self.processEvents(0.01)
        selected_channel_value = plot.selected_channel_value.grab()
        plot_graphics = plot.plot.grab()

        # Evaluate
        self.assertTrue(Pixmap.has_color(selected_channel_value, red))
        self.assertTrue(Pixmap.has_color(selected_channel_value, green))
        self.assertFalse(Pixmap.has_color(selected_channel_value, self.channels[0]))
        self.assertTrue(Pixmap.has_color(plot_graphics, red))
        self.assertFalse(Pixmap.has_color(plot_graphics, self.channels[0]))

        x, y, w, h = sig.timestamps[med_], sig.samples[med_], sig.timestamps[max_], sig.samples[max_]
        # Set X and Y ranges for viewbox
        plot.plot.viewbox.setXRange(x, w, padding=0)
        plot.plot.viewbox.setYRange(y, h, padding=0)
        self.processEvents(0.1)

        # Click in the middle of the plot
        QTest.mouseClick(
            plot.plot.viewport(),
            Qt.MouseButton.LeftButton,
            Qt.KeyboardModifier.NoModifier,
            QPoint(plot.plot.width() // 2, plot.plot.height() // 2),
        )
        self.processEvents(1)
        selected_channel_value = plot.selected_channel_value.grab()
        plot_graphics = plot.plot.grab()

        # Evaluate
        self.assertFalse(Pixmap.has_color(selected_channel_value, red))
        self.assertFalse(Pixmap.has_color(selected_channel_value, green))
        self.assertFalse(Pixmap.has_color(plot_graphics, red))
        self.assertTrue(Pixmap.has_color(selected_channel_value, self.channels[0]))
        self.assertTrue(Pixmap.has_color(plot_graphics, self.channels[0]))



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'error_dialog.ui'
##
## Created by: Qt User Interface Compiler version 6.7.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QDialog, QHBoxLayout, QLabel,
    QPushButton, QSizePolicy, QSpacerItem, QVBoxLayout,
    QWidget)
from . import resource_rc

class Ui_ErrorDialog(object):
    def setupUi(self, ErrorDialog):
        if not ErrorDialog.objectName():
            ErrorDialog.setObjectName(u"ErrorDialog")
        ErrorDialog.resize(622, 114)
        icon = QIcon()
        icon.addFile(u":/error.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        ErrorDialog.setWindowIcon(icon)
        ErrorDialog.setSizeGripEnabled(True)
        self.layout = QVBoxLayout(ErrorDialog)
        self.layout.setSpacing(1)
        self.layout.setObjectName(u"layout")
        self.layout.setContentsMargins(1, 1, 1, 1)
        self.error_message = QLabel(ErrorDialog)
        self.error_message.setObjectName(u"error_message")

        self.layout.addWidget(self.error_message)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(1)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.horizontalLayout_2.addItem(self.verticalSpacer)


        self.layout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(1)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.show_trace_btn = QPushButton(ErrorDialog)
        self.show_trace_btn.setObjectName(u"show_trace_btn")

        self.horizontalLayout.addWidget(self.show_trace_btn)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)

        self.copy_to_clipboard_btn = QPushButton(ErrorDialog)
        self.copy_to_clipboard_btn.setObjectName(u"copy_to_clipboard_btn")

        self.horizontalLayout.addWidget(self.copy_to_clipboard_btn)

        self.horizontalLayout.setStretch(0, 1)

        self.layout.addLayout(self.horizontalLayout)

        self.status = QLabel(ErrorDialog)
        self.status.setObjectName(u"status")

        self.layout.addWidget(self.status)

        self.layout.setStretch(1, 1)

        self.retranslateUi(ErrorDialog)

        QMetaObject.connectSlotsByName(ErrorDialog)
    # setupUi

    def retranslateUi(self, ErrorDialog):
        ErrorDialog.setWindowTitle(QCoreApplication.translate("ErrorDialog", u"Dialog", None))
        self.error_message.setText("")
        self.show_trace_btn.setText(QCoreApplication.translate("ErrorDialog", u"Show error trace", None))
        self.copy_to_clipboard_btn.setText(QCoreApplication.translate("ErrorDialog", u"Copy to clipboard", None))
        self.status.setText("")
    # retranslateUi



import os
import re
import sys
from traceback import format_exc

from natsort import natsorted
from PySide6 import QtWidgets

from asammdf.gui.utils import excepthook

from ..ui.simple_search_dialog import Ui_SimpleSearchDialog

sys.excepthook = excepthook


class SimpleSearch(Ui_SimpleSearchDialog, QtWidgets.QDialog):
    def __init__(self, channels_db, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setupUi(self)

        self.result = set()
        self.channels_db = channels_db

        self.apply_btn.clicked.connect(self._apply)
        self.add_btn.clicked.connect(self._add)
        self.cancel_btn.clicked.connect(self._cancel)

        self.search_box.editingFinished.connect(self.search_text_changed)
        self.match_kind.currentTextChanged.connect(self.search_box.textChanged.emit)

        self.matches.itemDoubleClicked.connect(self._match_double_clicked)
        self.matches.itemSelectionChanged.connect(self.show_match_comment)
        self.selection.itemDoubleClicked.connect(self._selection_double_clicked)

        self.setWindowTitle("Search channels")

        self.search_box.setFocus()

    def show_match_comment(self, items=None):
        items = self.matches.selectedItems()
        if items:
            item = items[0]
            comment = self.channels_db[item.text(0)]
            self.comment.setText(comment)
        else:
            self.comment.setText("")

    def _match_double_clicked(self, new_item):
        selection = set()

        iterator = QtWidgets.QTreeWidgetItemIterator(self.selection)
        while item := iterator.value():
            selection.add(item.text(0))

            iterator += 1

        if new_item.text(0) not in selection:
            selection.add(new_item.text(0))

            selection = natsorted(selection)

            items = [QtWidgets.QTreeWidgetItem([name]) for name in selection]

            self.selection.clear()
            self.selection.addTopLevelItems(items)

    def _selection_double_clicked(self, item):
        root = self.selection.invisibleRootItem()
        (item.parent() or root).removeChild(item)

    def search_text_changed(self):
        self.comment.setText("")
        text = self.search_box.text().strip()
        if len(text) >= 2:
            if self.match_kind.currentText() == "Wildcard":
                wildcard = f"{os.urandom(6).hex()}_WILDCARD_{os.urandom(6).hex()}"
                pattern = text.replace("*", wildcard)
                pattern = re.escape(pattern)
                pattern = pattern.replace(wildcard, ".*")
            else:
                pattern = text

            try:
                if self.case_sensitivity.currentText() == "Case insensitive":
                    pattern = re.compile(f"(?i){pattern}")
                else:
                    pattern = re.compile(pattern)

                matches = []
                for name in self.channels_db:
                    if pattern.fullmatch(name):
                        matches.append(name)

                items = [QtWidgets.QTreeWidgetItem([name]) for name in matches]

                self.matches.clear()
                self.matches.addTopLevelItems(items)

                if items:
                    self.status.setText("")
                else:
                    self.status.setText("No match found")

                self.matches.setFocus()
            except:
                self.status.setText(format_exc())
                self.matches.clear()

    def _add(self, event):
        items = set()

        iterator = QtWidgets.QTreeWidgetItemIterator(self.selection)
        while item := iterator.value():
            items.add(item.text(0))
            iterator += 1

        for item in self.matches.selectedItems():
            items.add(item.text(0))

        items = natsorted(items)

        items = [QtWidgets.QTreeWidgetItem([name]) for name in items]

        self.selection.clear()
        self.selection.addTopLevelItems(items)

    def _apply(self, event):
        self.result = set()
        iterator = QtWidgets.QTreeWidgetItemIterator(self.selection)
        while item := iterator.value():
            self.result.add(item.text(0))
            iterator += 1
        self.close()

    def _cancel(self, event):
        self.result = set()
        self.close()



import numpy as np


def Function1(t=0):
    sequence = range(10)
    filtered = list(filter(lambda i: i % 2, sequence))
    if filtered:
        for _ in filtered:
            if not _:
                continue
            try:
                int(_)
            except ValueError:
                print(str(_))
            break
        else:
            return 1
        return 2
    else:
        return 0


def Function2(t=0):
    sequence = range(10)
    filtered = filter(lambda i: i % 2, sequence)
    if len(filtered):
        for _ in filtered:
            if not _:
                continue
            try:
                int(_)
            except ValueError:
                print(str(_))
            break
        else:
            return 1
    else:
        return 0


def UnresolvedVariable(t=0):
    a, b = 0, 1
    if a:
        c = 0
    return c


def WrongDefinition():
    return 1


def gray2dec(position_sensor_value=0, t=0):
    for shift in (8, 4, 2, 1):
        position_sensor_value = position_sensor_value ^ (position_sensor_value >> shift)

    return position_sensor_value


def maximum(channel1=0, channel2=-1, channel3=0, t=0):
    return max(channel1, channel2, channel3)


def rpm_to_rad_per_second(speed=0, t=0):
    return 2 * np.pi * speed / 60



import numpy as np
from numpy.typing import NDArray
from typing_extensions import Any

from ..signal import InvalidationArray
from .blocks_common import UnpackFrom
from .utils import DataBlockInfo

def sort_data_block(
    signal_data: bytes,
    partial_records: dict[int, list[bytes]],
    record_size: dict[int, int],
    id_size: int,
    optional: UnpackFrom[tuple[int]] | None,
) -> bytes: ...
def extract(signal_data: bytes, is_byte_array: bool, offsets: NDArray[np.uintp]) -> NDArray[np.uint8]: ...
def get_vlsd_max_sample_size(data: bytes, offsets: NDArray[np.uint64], count: int) -> int: ...
def get_channel_raw_bytes(
    data_block: bytes | bytearray, record_size: int, byte_offset: int, byte_count: int
) -> bytearray: ...
def get_invalidation_bits_array(
    data_block: bytes | bytearray, invalidation_size: int, invalidation_pos: int
) -> NDArray[np.uint8]: ...
def get_channel_raw_bytes_parallel(
    data_block: bytes | bytearray, record_size: int, signals: list[list[int]], thread_count: int = 11
) -> list[bytearray]: ...
def data_block_from_arrays(
    data_blocks: list[tuple[bytes | NDArray[Any], int]], cycles_obj: int, thread_count: int = 11
) -> bytearray: ...
def bytes_dtype_size(ret: NDArray[Any]) -> int: ...
def get_channel_raw_bytes_complete(
    data_blocks_info: list[DataBlockInfo],
    signals: list[tuple[int, int, int]],
    file_name: str,
    cycles: int,
    record_size: int,
    invalidation_bytes: int,
    group_index: int,
    thread_count: int = 11,
) -> tuple[tuple[bytearray, InvalidationArray | None], ...]: ...



#!/usr/bin/env python
from unittest import mock

from PySide6 import QtCore, QtGui, QtTest, QtWidgets

from test.asammdf.gui.test_base import DragAndDrop
from test.asammdf.gui.widgets.test_BasePlotWidget import TestPlotWidget


class TestDragAndDrop(TestPlotWidget):
    # Note: Test Plot Widget through FileWidget.

    def setUp(self):
        super().setUp()

        self.setUpFileWidget(measurement_file=self.measurement_file, default=True)
        # Switch ComboBox to "Natural sort"
        self.widget.channel_view.setCurrentText("Natural sort")

        self.create_window(window_type="Plot")
        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 1)

    def test_Plot_ChannelSelection_DragAndDrop_fromFile_toPlot_0(self):
        """
        Test Scope:
            - Test DragAndDrop Action from FileWidget.channel_tree to Plot.channel_selection and to Plot.plot.
            - Channels are selection one by one.
            - Ensure that Drag and Drop Action allow channels to be added to Plot.channel_selection
        Precondition:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
        Events:
            - Drag and Drop channel from FileWidget.channel_tree to Plot
            - Drag and Drop same channel from FileWidget.channel_tree to Plot
        Evaluate:
            - Evaluate that two channels are added to Plot "channel_selection"
        """
        # Event
        # Select channel
        channel_tree = self.widget.channels_tree
        plot = self.widget.mdi_area.subWindowList()[0].widget()
        item = channel_tree.topLevelItem(0)
        item.setSelected(True)
        channel_name = item.text(self.Column.NAME)
        drag_position = channel_tree.visualItemRect(item).center()
        drop_position = plot.channel_selection.viewport().rect().center()

        # PreEvaluation
        self.assertEqual(0, plot.channel_selection.topLevelItemCount())
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=plot.channel_selection,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        # Evaluate
        plot_channel = plot.channel_selection.topLevelItem(0)
        plot_channel_name = plot_channel.text(self.Column.NAME)
        self.assertEqual(1, plot.channel_selection.topLevelItemCount())
        self.assertEqual(channel_name, plot_channel_name)

        drop_position = plot.plot.viewport().rect().center()
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=plot.plot,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        self.assertEqual(2, plot.channel_selection.topLevelItemCount())

    def test_Plot_ChannelSelection_DragAndDrop_fromFile_toPlot_1(self):
        """
        Test Scope:
            - Test DragAndDrop Action from FileWidget.channel_tree to Plot.channel_selection and to Plot.plot.
            - Multiple Channels are selected and dragged.
            - Ensure that Drag and Drop Action allow channels to be added to Plot.channel_selection
        Precondition:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
        Events:
            - Select 3 channels from FileWidget.channel_tree
            - Drag and Drop channels from FileWidget.channel_tree to Plot
        Evaluate:
            - Evaluate that 3 channels are added to Plot "channel_selection"
        """
        # Event
        channel_tree = self.widget.channels_tree
        plot = self.widget.mdi_area.subWindowList()[0].widget()

        # Select 3 from channels but not from beginning
        # Avoid $ChannelLog because it's empty channel
        iterator = QtWidgets.QTreeWidgetItemIterator(channel_tree)
        count = 6
        selected_channels = []
        item = None
        while iterator.value() and count:
            count -= 1
            if count > 2:
                iterator += 1
                continue
            item = iterator.value()
            item.setSelected(True)
            selected_channels.append(item.text(0))
            iterator += 1

        drag_position = channel_tree.visualItemRect(item).center()
        drop_position = plot.channel_selection.viewport().rect().center()

        # PreEvaluation
        self.assertEqual(0, plot.channel_selection.topLevelItemCount())
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=plot,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        # Evaluate
        self.assertEqual(3, plot.channel_selection.topLevelItemCount())
        iterator = QtWidgets.QTreeWidgetItemIterator(plot.channel_selection)
        plot_channels = []
        while iterator.value():
            item = iterator.value()
            plot_channels.append(item.text(self.Column.NAME))
            iterator += 1
        self.assertListEqual(selected_channels, plot_channels)

    def test_Plot_ChannelSelection_DragAndDrop_fromPlotCS_toSamePlotCS(self):
        """
        Test Scope:
            - Test DragAndDrop Action from Plot.channel_selection to Plot.channel_selection
            - Case 0: Channels are selected one by one.
            - Case 1: Multiple Channels are selected.
            - Case 2: Create Channel Group. Drag channels inside the group one by one
            - Case 3: Create Channel Group. Drag multiple channels at once inside the group.
            - Case 4: Drag Group inside the Group
            - Case 5: Drag Group outside the Group
            - Ensure that Drag and Drop Action allow channels to be sorted to Plot.channel_selection
        Precondition:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
        Events:
            - Select 5 channels and DragAndDrop them to Plot.channel_selection.
                - One channel should be duplicated.
            - Case 0:
                - DragAndDrop first channel to 3rd position.
            - Case 1:
                - DragAndDrop 2nd and 3rd channels on last position.
            - Case 2:
                - Create ChannelGroup
                - DragAndDrop first channel inside the group.
                - DragAndDrop 2nd channel inside the group.
            - Case 3:
                - Create ChannelGroup
                - DragAndDrop last two channels inside the group.
            - Case 4:
                - Drag Group inside the Group
            - Case 5:
                - Drag Group outside the Group
        Evaluate:
            - Case 0:
                - Evaluate that channel changed the position in the tree.
            - Case 1:
                - Evaluate that channels changed their position in the tree.
            - Case 2:
                - Evaluate that channel group was created. New item is present on the tree.
                - Evaluate that channel was moved as child to this item. Item should no longer exist outside the group.
                - Evaluate that duplicated channel was moved inside the group, and there is still one more wo parent
                 on the tree
            - Case 3:
                - Evaluate that channels were moved inside the group and does no longer exist outside.
            - Case 4:
                - Evaluate that group was moved inside the group.
            - Case 5:
                - Evaluate that group was moved outside the group.
        """
        # Event
        channel_tree = self.widget.channels_tree
        plot = self.widget.mdi_area.subWindowList()[0].widget()

        # Select 5 from channels but not from beginning
        # Avoid $ChannelLog because it's empty channel
        iterator = QtWidgets.QTreeWidgetItemIterator(channel_tree)
        count = -1
        selected_channels = []
        drag_item = None
        while iterator.value():
            count += 1
            # Skip over first 3
            if count < 2:
                iterator += 1
                continue
            item = iterator.value()
            if item and count < 7:
                drag_item = item
                item.setSelected(True)
                item.setCheckState(0, QtCore.Qt.CheckState.Checked)
                selected_channels.append(item.text(0))
            iterator += 1

        drag_position = channel_tree.visualItemRect(drag_item).center()
        drop_position = plot.channel_selection.viewport().rect().center()

        # PreEvaluation
        self.assertEqual(0, plot.channel_selection.topLevelItemCount())
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=plot,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        self.assertEqual(5, plot.channel_selection.topLevelItemCount())

        # Add one channel twice
        # Switch ComboBox to "Selected channels only"
        self.widget.channel_view.setCurrentText("Selected channels only")
        duplicated_channel = channel_tree.topLevelItem(0)
        duplicated_channel.setSelected(True)
        drag_position = channel_tree.visualItemRect(duplicated_channel).center()
        duplicated_channel = duplicated_channel.text(0)
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=plot,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        self.assertEqual(6, plot.channel_selection.topLevelItemCount())

        # Case 0:
        with self.subTest("test_test_Plot_ChannelSelection_DragAndDrop_fromPlotCS_toPlot_0"):
            # DragAndDrop first channel to 3rd position.
            first_channel = plot.channel_selection.topLevelItem(0)
            third_channel = plot.channel_selection.topLevelItem(2)
            self.move_item_inside_channels_tree_widget(src=first_channel, dst=third_channel)
            # Get Names
            first_channel = first_channel.text(self.Column.NAME)
            third_channel = third_channel.text(self.Column.NAME)

            # Evaluate
            # First channel position was changed.
            new_first_channel = plot.channel_selection.topLevelItem(0).text(self.Column.NAME)
            self.assertNotEqual(first_channel, new_first_channel)
            # Evaluate that first channel was moved to third channel position.
            new_third_channel = plot.channel_selection.topLevelItem(2).text(self.Column.NAME)
            new_fourth_channel = plot.channel_selection.topLevelItem(1).text(self.Column.NAME)
            self.assertEqual(first_channel, new_third_channel)
            self.assertEqual(third_channel, new_fourth_channel)

        # Case 1:
        with self.subTest("test_test_Plot_ChannelSelection_DragAndDrop_fromPlotCS_toPlot_1"):
            # DragAndDrop 2nd and 3rd channels on last position.
            second_channel = plot.channel_selection.topLevelItem(1)
            third_channel = plot.channel_selection.topLevelItem(2)
            last_channel = plot.channel_selection.topLevelItem(5)
            # Select
            second_channel.setSelected(True)
            third_channel.setSelected(True)
            self.move_item_inside_channels_tree_widget(src=third_channel, dst=last_channel)
            # Get Names
            second_channel = second_channel.text(self.Column.NAME)
            third_channel = third_channel.text(self.Column.NAME)

            # Evaluate
            new_second_channel = plot.channel_selection.topLevelItem(1).text(self.Column.NAME)
            new_third_channel = plot.channel_selection.topLevelItem(2).text(self.Column.NAME)
            self.assertNotEqual(second_channel, new_second_channel)
            self.assertNotEqual(third_channel, new_third_channel)
            new_fifth_channel = plot.channel_selection.topLevelItem(4).text(self.Column.NAME)
            new_sixth_channel = plot.channel_selection.topLevelItem(5).text(self.Column.NAME)
            self.assertEqual(second_channel, new_fifth_channel)
            self.assertEqual(third_channel, new_sixth_channel)

        # Case 2:
        with self.subTest("test_test_Plot_ChannelSelection_DragAndDrop_fromPlotCS_toPlot_2"):
            # Create Channel Group. Drag channels inside the group one by one
            with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QInputDialog.getText") as mc_getText:
                # Create Channel Group
                mc_getText.return_value = "FirstGroup", True
                QtTest.QTest.keySequence(plot.channel_selection, QtGui.QKeySequence("Shift+Insert"))
                # PreEvaluation: Check if there is one extra-item
                self.assertEqual(7, plot.channel_selection.topLevelItemCount())
                # Get Group Position
                first_group = None
                for index in range(plot.channel_selection.topLevelItemCount()):
                    item = plot.channel_selection.topLevelItem(index)
                    if item.text(self.Column.NAME) == "FirstGroup":
                        first_group = item
                        break
                else:
                    self.fail("FirstGroup is not present on Plot Channel Selection.")
                first_group.setExpanded(True)
                # Get First Item that will be moved
                first_channel = plot.channel_selection.topLevelItem(1)
                if first_channel.text(self.Column.NAME) == duplicated_channel:
                    first_channel = plot.channel_selection.topLevelItem(2)
                # Get the Name of first channel
                first_channel_ = first_channel
                first_channel = first_channel.text(self.Column.NAME)

                # PreEvaluation: Ensure that group has no child
                self.assertEqual(0, first_group.childCount())
                self.move_item_inside_channels_tree_widget(src=first_channel_, dst=first_group)

                # Evaluate
                self.assertEqual(1, first_group.childCount())
                self.assertEqual(first_channel, first_group.child(0).text(self.Column.NAME))
                self.assertEqual(6, plot.channel_selection.topLevelItemCount())

                second_channel = None
                for index in range(plot.channel_selection.topLevelItemCount()):
                    item = plot.channel_selection.topLevelItem(index)
                    if item.text(self.Column.NAME) == duplicated_channel:
                        second_channel = item
                        break
                else:
                    self.fail("Duplicate Channel is not found anymore.")
                self.move_item_inside_channels_tree_widget(src=second_channel, dst=first_group)

                # Evaluate
                self.assertEqual(2, first_group.childCount())
                self.assertEqual(duplicated_channel, first_group.child(1).text(self.Column.NAME))
                self.assertEqual(5, plot.channel_selection.topLevelItemCount())

        # Case 3:
        with self.subTest("test_test_Plot_ChannelSelection_DragAndDrop_fromPlotCS_toPlot_3"):
            # Create Channel Group. Drag multiple channels inside the group
            with mock.patch("asammdf.gui.widgets.tree.QtWidgets.QInputDialog.getText") as mc_getText:
                # Create Channel Group
                mc_getText.return_value = "SecondGroup", True
                QtTest.QTest.keySequence(plot.channel_selection, QtGui.QKeySequence("Shift+Insert"))
                # PreEvaluation: Check if there is one extra-item
                self.assertEqual(6, plot.channel_selection.topLevelItemCount())
                # Get Group Position
                second_group = None
                for index in range(plot.channel_selection.topLevelItemCount()):
                    item = plot.channel_selection.topLevelItem(index)
                    if item.text(self.Column.NAME) == "SecondGroup":
                        second_group = item
                        break
                else:
                    self.fail("SecondGroup is not present on Plot Channel Selection.")
                second_group.setExpanded(True)
                # Get Channels
                last_channel_0 = plot.channel_selection.topLevelItem(plot.channel_selection.topLevelItemCount() - 1)
                last_channel_1 = plot.channel_selection.topLevelItem(plot.channel_selection.topLevelItemCount() - 2)
                last_channel_0.setSelected(True)
                last_channel_1.setSelected(True)
                self.move_item_inside_channels_tree_widget(src=last_channel_1, dst=second_group)

                # Evaluate
                self.assertEqual(2, second_group.childCount())
                self.assertEqual(4, plot.channel_selection.topLevelItemCount())

        # Case 4:
        with self.subTest("test_test_Plot_ChannelSelection_DragAndDrop_fromPlotCS_toPlot_4"):
            # Drag Group inside the Group
            # Get Group Positions
            first_group, second_group = None, None
            for index in range(plot.channel_selection.topLevelItemCount()):
                item = plot.channel_selection.topLevelItem(index)
                if item.text(self.Column.NAME) == "FirstGroup":
                    first_group = item
                elif item.text(self.Column.NAME) == "SecondGroup":
                    second_group = item
                if first_group and second_group:
                    break
            else:
                self.fail("Groups are not present on Plot Channel Selection.")
            self.move_item_inside_channels_tree_widget(src=second_group, dst=first_group)

            # Evaluate
            self.assertEqual(3, first_group.childCount())
            self.assertEqual(3, plot.channel_selection.topLevelItemCount())

        # Case 5:
        with self.subTest("test_test_Plot_ChannelSelection_DragAndDrop_fromPlotCS_toPlot_5"):
            # Drag Group outside the Group
            # Get Group Positions
            first_group, second_group = None, None
            for index in range(plot.channel_selection.topLevelItemCount()):
                item = plot.channel_selection.topLevelItem(index)
                if item.text(self.Column.NAME) == "FirstGroup":
                    first_group = item
                    break
            else:
                self.fail("FirstGroup is not present on Plot Channel Selection.")
            for index in range(first_group.childCount()):
                item = first_group.child(index)
                if item.text(self.Column.NAME) == "SecondGroup":
                    second_group = item
                    break
            else:
                self.fail("SecondGroup is not present on Plot Channel Selection.")
            self.move_item_inside_channels_tree_widget(src=second_group, dst=plot.channel_selection)

            # Evaluate
            self.assertEqual(2, first_group.childCount())
            self.assertEqual(4, plot.channel_selection.topLevelItemCount())

    def test_Plot_ChannelSelection_DragAndDrop_fromPlot_toPlot(self):
        """
        Test Scope: Validate that channels can be dragged and dropped between Plot windows. (Ex: from Plot 0 to Plot 1)
        Precondition:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as a window type.
        Events:
            - Select one channel and drag it to the 'Plot 0'
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
            - Select one channel and drag it to the 'Plot 1'
            - Select channel from 'Plot 0' and drag it to the 'Plot 1'
        Evaluate:
            - Validate that channel from 'Plot 0' is added to 'Plot 1'
        """
        # Event
        channel_tree = self.widget.channels_tree
        plot_0 = self.widget.mdi_area.subWindowList()[0].widget()
        # Random Channels
        channel_0 = channel_tree.topLevelItem(8)
        channel_1 = channel_tree.topLevelItem(13)

        # Drag one Channel from FileWidget channel_tree to Plot_0
        drag_position = channel_tree.visualItemRect(channel_0).center()
        drop_position = plot_0.channel_selection.viewport().rect().center()
        self.mouseClick_WidgetItem(channel_0)
        # PreEvaluation
        self.assertEqual(0, plot_0.channel_selection.topLevelItemCount())
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=plot_0,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        self.assertEqual(1, plot_0.channel_selection.topLevelItemCount())

        # Create New Plot Window
        self.create_window(window_type="Plot")
        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 2)

        plot_1 = self.widget.mdi_area.subWindowList()[1].widget()

        # Drag one Channel from FileWidget channel_tree to Plot_0
        drag_position = channel_tree.visualItemRect(channel_1).center()
        drop_position = plot_1.channel_selection.viewport().rect().center()
        self.mouseClick_WidgetItem(channel_1)
        # PreEvaluation
        self.assertEqual(0, plot_1.channel_selection.topLevelItemCount())
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=plot_1,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        self.assertEqual(1, plot_1.channel_selection.topLevelItemCount())

        # Select the channel from 'Plot 0' and drag it to 'Plot 1'
        # Drag one Channel from FileWidget channel_tree to Plot_0
        plot_0_channel = plot_0.channel_selection.topLevelItem(0)
        plot_0_channel_name = plot_0_channel.text(self.Column.NAME)
        drag_position = plot_0.channel_selection.visualItemRect(plot_0_channel).center()
        drop_position = plot_1.channel_selection.viewport().rect().center()
        self.mouseClick_WidgetItem(plot_0_channel)
        # PreEvaluation
        self.assertEqual(1, plot_1.channel_selection.topLevelItemCount())
        DragAndDrop(
            src_widget=plot_0.channel_selection,
            dst_widget=plot_1,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        # Evaluate
        plot_1_new_channel = plot_1.channel_selection.topLevelItem(1)
        plot_1_new_channel_name = plot_1_new_channel.text(self.Column.NAME)
        self.assertEqual(2, plot_1.channel_selection.topLevelItemCount())
        self.assertEqual(plot_0_channel_name, plot_1_new_channel_name)

    def test_Plot_ChannelSelection_DragAndDrop_fromNumeric_toPlot(self):
        """
        Test Scope: Validate that channels can be dragged and dropped between Plot window and Numeric.
        Ex: from 'Numeric 0' to 'Plot 0'
        Precondition:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
                - Simulate that Plot window is selected as window type.
        Events:
            - Select one channel and drag it to the 'Plot 0'
            - Press PushButton "Create Window"
                - Simulate that Numeric window is selected as window type.
            - Select one channel and drag it to the 'Numeric 0'
            - Select channel from 'Numeric 0' and drag it to the 'Plot 0'
        Evaluate:
            - Validate that channel from 'Numeric 0' is added to 'Plot 0'
        """
        # Event
        channel_tree = self.widget.channels_tree
        plot_0 = self.widget.mdi_area.subWindowList()[0].widget()
        # Random Channels
        channel_0 = channel_tree.topLevelItem(8)
        channel_1 = channel_tree.topLevelItem(0)

        # Drag one Channel from FileWidget channel_tree to Plot_0
        drag_position = channel_tree.visualItemRect(channel_0).center()
        drop_position = plot_0.channel_selection.viewport().rect().center()
        self.mouseClick_WidgetItem(channel_0)
        # PreEvaluation
        self.assertEqual(0, plot_0.channel_selection.topLevelItemCount())
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=plot_0,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        self.assertEqual(1, plot_0.channel_selection.topLevelItemCount())

        # Create New Numeric Window
        self.create_window(window_type="Numeric")
        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 2)

        numeric = self.widget.mdi_area.subWindowList()[1].widget()

        # Tile Horizontally
        QtTest.QTest.keySequence(plot_0.plot, "Shift+H")

        # Drag one Channel from FileWidget channel_tree to Numeric_0
        drag_position = channel_tree.visualItemRect(channel_1).center()
        drop_position = numeric.channels.dataView.viewport().rect().center()
        self.mouseClick_WidgetItem(channel_1)
        DragAndDrop(
            src_widget=channel_tree,
            dst_widget=numeric.channels.dataView,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        # Evaluate
        numeric_data = numeric.channels.dataView
        # Select first row
        numeric_data.selectRow(0)
        numeric_channel = numeric_data.model().data(numeric_data.model().index(0, 0))
        self.assertEqual(channel_1.text(self.Column.NAME), numeric_channel)

        # Drag one Channel from Numeric_0 to Plot_0
        index = numeric_data.model().index(0, 0)
        drag_position = numeric_data.visualRect(index).center()
        drop_position = plot_0.channel_selection.viewport().rect().center()

        DragAndDrop(
            src_widget=numeric_data,
            dst_widget=plot_0.channel_selection,
            src_pos=drag_position,
            dst_pos=drop_position,
        )
        # Evaluate
        plot_0_new_channel = plot_0.channel_selection.topLevelItem(1)
        plot_0_new_channel_name = plot_0_new_channel.text(self.Column.NAME)
        self.assertEqual(2, plot_0.channel_selection.topLevelItemCount())
        self.assertEqual(numeric_channel, plot_0_new_channel_name)



#!/usr/bin/env python

from PySide6 import QtCore, QtTest
from PySide6.QtWidgets import QPushButton as QBtn
from PySide6.QtWidgets import QTreeWidgetItemIterator

from asammdf.gui.serde import COLORS
from asammdf.gui.utils import BLUE
from test.asammdf.gui.test_base import Pixmap
from test.asammdf.gui.widgets.test_BasePlotWidget import TestPlotWidget


class TestPushButtons(TestPlotWidget):
    def setUp(self):
        super().setUp()

        self.channel_0_name = "ASAM_[14].M.MATRIX_DIM_16.UBYTE.IDENTICAL"
        self.channel_1_name = "ASAM_[15].M.MATRIX_DIM_16.UBYTE.IDENTICAL"

        # Open test file in File Widget
        self.setUpFileWidget(measurement_file=self.measurement_file, default=True)

        # Switch ComboBox to "Natural sort"
        self.widget.channel_view.setCurrentText("Natural sort")
        # Press PushButton "Create Window"
        self.create_window(window_type="Plot")
        self.assertEqual(len(self.widget.mdi_area.subWindowList()), 1)
        self.plot = self.widget.mdi_area.subWindowList()[0].widget()

        # Press PushButton "Hide axis"
        if not self.plot.hide_axes_btn.isFlat():
            QtTest.QTest.mouseClick(self.plot.hide_axes_btn, QtCore.Qt.MouseButton.LeftButton)

        # Press PushButton "Hide bookmarks"
        if not self.plot.bookmark_btn.isFlat():
            QtTest.QTest.mouseClick(self.plot.bookmark_btn, QtCore.Qt.MouseButton.LeftButton)

        if not self.plot.focused_mode_btn.isFlat():
            QtTest.QTest.mouseClick(self.plot.focused_mode_btn, QtCore.Qt.MouseButton.LeftButton)

        self.processEvents(0.1)
        # Save PixMap of clear plot
        clear_pixmap = self.plot.plot.viewport().grab()
        self.assertTrue(Pixmap.is_black(clear_pixmap))

        # Add Channels to Plot
        self.add_channels([self.channel_0_name, self.channel_1_name])
        self.plot_tree_channel_0 = self.channels[0]
        self.plot_tree_channel_1 = self.channels[1]
        self.assertEqual(2, self.plot.channel_selection.topLevelItemCount())

        # Identify PlotSignal
        self.plot_graph_channel_0, self.plot_graph_channel_1 = None, None
        for channel in self.plot.plot.signals:
            if channel.name == self.channel_0_name:
                self.plot_graph_channel_0 = channel
            elif channel.name == self.channel_1_name:
                self.plot_graph_channel_1 = channel

    def test_Plot_ChannelSelection_PushButton_ValuePanel(self):
        """
        Test Scope:
            Check that Value Panel label is visible or hidden according to Push Button
            Check that Value Panel label is updated according signal samples
        Precondition:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
            - Drag and Drop 2 channels from FileWidget.channels_tree to Plot.channels_selection
        Events:
            - Press PushButton "Show selected channel value panel"
            - Press PushButton "Hide selected channel value panel"
            - Press PushButton "Show selected channel value panel"
            - Select one channel
            - Use navigation keys to change values
            - Select 2nd channel
            - Use navigation keys to change values
        Evaluate:
            - Evaluate that label is visible when button "Show selected channel value panel" is pressed.
            - Evaluate that label is hidden when button "Hide selected channel value panel" is pressed.
            - Evaluate that value from label is equal with: Channel Value + Channel Unit
            - Evaluate that value is updated according channel selected and current value
        """
        # Event
        if self.plot.selected_channel_value.isVisible():
            # Press PushButton "Hide selected channel value panel"
            QtTest.QTest.mouseClick(self.plot.selected_channel_value_btn, QtCore.Qt.MouseButton.LeftButton)
        # Press PushButton "Show selected channel value panel"
        QtTest.QTest.mouseClick(self.plot.selected_channel_value_btn, QtCore.Qt.MouseButton.LeftButton)
        self.assertTrue(self.plot.selected_channel_value.isVisible())

        # Press PushButton "Hide selected channel value panel"
        QtTest.QTest.mouseClick(self.plot.selected_channel_value_btn, QtCore.Qt.MouseButton.LeftButton)
        self.assertFalse(self.plot.selected_channel_value.isVisible())
        # Press PushButton "Show selected channel value panel"
        QtTest.QTest.mouseClick(self.plot.selected_channel_value_btn, QtCore.Qt.MouseButton.LeftButton)
        self.assertTrue(self.plot.selected_channel_value.isVisible())

        # Select Channel
        self.mouseClick_WidgetItem(self.plot_tree_channel_0)

        # Evaluate
        plot_channel_0_value = self.plot_tree_channel_0.text(self.Column.VALUE)
        plot_channel_0_unit = self.plot_tree_channel_0.text(self.Column.UNIT)
        self.assertEqual(
            f"{plot_channel_0_value} {plot_channel_0_unit}",
            self.plot.selected_channel_value.text(),
        )

        # Event
        self.plot.plot.setFocus()
        self.processEvents(0.1)
        # Send Key strokes
        for _ in range(6):
            QtTest.QTest.keyClick(self.plot.plot, QtCore.Qt.Key.Key_Right)
            self.processEvents(0.1)
        self.processEvents(0.1)

        # Evaluate
        plot_channel_0_value = self.plot_tree_channel_0.text(self.Column.VALUE)
        plot_channel_0_unit = self.plot_tree_channel_0.text(self.Column.UNIT)
        self.assertEqual(
            f"{plot_channel_0_value} {plot_channel_0_unit}",
            self.plot.selected_channel_value.text(),
        )

        # Select 2nd Channel
        self.mouseClick_WidgetItem(self.plot_tree_channel_1)

        # Evaluate
        plot_channel_1_value = self.plot_tree_channel_1.text(self.Column.VALUE)
        plot_channel_1_unit = self.plot_tree_channel_1.text(self.Column.UNIT)
        self.assertEqual(
            f"{plot_channel_1_value} {plot_channel_1_unit}",
            self.plot.selected_channel_value.text(),
        )

        # Event
        self.plot.plot.setFocus()
        self.processEvents(0.1)
        # Send Key strokes
        for _ in range(6):
            QtTest.QTest.keyClick(self.plot.plot, QtCore.Qt.Key.Key_Right)
            self.processEvents(0.1)
        self.processEvents(0.1)

        # Evaluate
        plot_channel_1_value = self.plot_tree_channel_1.text(self.Column.VALUE)
        plot_channel_1_unit = self.plot_tree_channel_1.text(self.Column.UNIT)
        self.assertEqual(
            f"{plot_channel_1_value} {plot_channel_1_unit}",
            self.plot.selected_channel_value.text(),
        )

    def test_Plot_ChannelSelection_PushButton_FocusedMode(self):
        """
        Test Scope:
            Check if Plot is cleared when no channel is selected.
            Check if Plot is showing all channels when Focus Mode is disabled.
            Check if Plot is showing only one channel when Focus Mode is enabled.
        Precondition:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
            - Press PushButton HideAxis (easy for evaluation)
            - Drag and Drop channels from FileWidget.channels_tree to Plot.channels_selection:
                # First
                - ASAM_[15].M.MATRIX_DIM_16.UBYTE.IDENTICAL
                # Second
                - ASAM_[14].M.MATRIX_DIM_16.UBYTE.IDENTICAL
        Events:
            - Press PushButton FocusMode
            - Press PushButton FocusMode
        Evaluate:
            - Evaluate that channels are displayed when FocusMode is disabled.
            - Evaluate that selected channels is displayed when FocusMode is enabled.
        """
        # Events
        if not self.plot.focused_mode_btn.isFlat():
            QtTest.QTest.mouseClick(self.plot.focused_mode_btn, QtCore.Qt.MouseButton.LeftButton)

        channels_present_pixmap = self.plot.plot.viewport().grab()
        self.assertFalse(Pixmap.is_black(pixmap=channels_present_pixmap))
        self.assertTrue(
            Pixmap.has_color(
                pixmap=channels_present_pixmap,
                color_name=self.plot_graph_channel_0.color_name,
            )
        )
        self.assertTrue(
            Pixmap.has_color(
                pixmap=channels_present_pixmap,
                color_name=self.plot_graph_channel_1.color_name,
            )
        )

        # Press Button Focus Mode
        QtTest.QTest.mouseClick(self.plot.focused_mode_btn, QtCore.Qt.MouseButton.LeftButton)

        # Evaluate
        focus_mode_clear_pixmap = self.plot.plot.viewport().grab()
        # No Channel is selected
        self.assertTrue(Pixmap.is_black(pixmap=focus_mode_clear_pixmap))

        # Select 2nd Channel
        QtTest.QTest.mouseClick(
            self.plot.channel_selection.viewport(),
            QtCore.Qt.MouseButton.LeftButton,
            QtCore.Qt.KeyboardModifier.NoModifier,
            self.plot.channel_selection.visualItemRect(self.plot_tree_channel_1).center(),
        )
        # Process flash until signal is present on plot.
        for _ in range(10):
            self.processEvents(timeout=0.01)
            focus_mode_channel_1_pixmap = self.plot.plot.viewport().grab()
            if Pixmap.has_color(
                pixmap=focus_mode_channel_1_pixmap,
                color_name=self.plot_graph_channel_1.color_name,
            ):
                break

        # Evaluate
        focus_mode_channel_1_pixmap = self.plot.plot.viewport().grab()
        self.assertFalse(Pixmap.is_black(pixmap=focus_mode_channel_1_pixmap))
        self.assertFalse(
            Pixmap.has_color(
                pixmap=focus_mode_channel_1_pixmap,
                color_name=self.plot_graph_channel_0.color_name,
            )
        )
        self.assertTrue(
            Pixmap.has_color(
                pixmap=focus_mode_channel_1_pixmap,
                color_name=self.plot_graph_channel_1.color_name,
            )
        )

    def test_Plot_ChannelSelection_PushButton_RegionDelta(self):
        """
        Test Scope:
            Check if computation is done when Region is selected.
        Events:
            - Open 'FileWidget' with valid measurement.
            - Switch ComboBox to "Natural sort"
            - Press PushButton "Create Window"
            - Press PushButton HideAxis (easy for evaluation)
            - Drag and Drop channels from FileWidget.channels_tree to Plot.channels_selection:
                # First
                - ASAM_[15].M.MATRIX_DIM_16.UBYTE.IDENTICAL
                # Second
                - ASAM_[14].M.MATRIX_DIM_16.UBYTE.IDENTICAL
            - Press PushButton Delta (range is not active)
            - Press Key 'R' for range selection
            - Press PushButton Delta (range is active)
            - Move cursors
            - Press Key 'R' for range selection
            - Press PushButton Delta (range is not active)
        Evaluate:
            - Evaluate that 'delta' char is present on channel values when range selection is active
            - Evaluate that 'delta' is correctly computed and result is updated on channel value
        """
        channel_0 = "ASAM_[14].M.MATRIX_DIM_16.UBYTE.IDENTICAL"
        channel_1 = "ASAM_[15].M.MATRIX_DIM_16.UBYTE.IDENTICAL"

        # Event
        if not self.plot.delta_btn.isFlat():
            QtTest.QTest.mouseClick(self.plot.delta_btn, QtCore.Qt.MouseButton.LeftButton)
            self.processEvents()

        # Ensure that delta char is not present on channel values
        self.assertNotIn("Δ", self.plot_tree_channel_0.text(self.Column.VALUE))
        self.assertNotIn("Δ", self.plot_tree_channel_1.text(self.Column.VALUE))

        # Press PushButton Delta (range is not active)
        QtTest.QTest.mouseClick(self.plot.delta_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()

        # Ensure that delta char is not present on channel values even if button is pressed
        self.assertNotIn("Δ", self.plot_tree_channel_0.text(self.Column.VALUE))
        self.assertNotIn("Δ", self.plot_tree_channel_1.text(self.Column.VALUE))

        # Press Key 'R' for range selection
        QtTest.QTest.keyClick(self.plot.plot, QtCore.Qt.Key.Key_R)
        self.processEvents(timeout=0.01)

        # Ensure that delta char is present on channel values even if button is pressed
        self.assertIn("Δ", self.plot_tree_channel_0.text(self.Column.VALUE))
        self.assertIn("Δ", self.plot_tree_channel_1.text(self.Column.VALUE))

        # Move cursor
        # Select channel_1
        QtTest.QTest.mouseClick(
            self.plot.channel_selection.viewport(),
            QtCore.Qt.MouseButton.LeftButton,
            QtCore.Qt.KeyboardModifier.NoModifier,
            self.plot.channel_selection.visualItemRect(self.plot_tree_channel_1).center(),
        )
        self.plot.plot.setFocus()
        self.processEvents(0.1)
        # Move a little bit in center of measurement
        for _ in range(15):
            QtTest.QTest.keyClick(self.plot.plot, QtCore.Qt.Key.Key_Right)
            self.processEvents(timeout=0.1)

        # Get current value: Ex: 'Δ = 8'. Get last number
        old_channel_0_value = int(self.plot_tree_channel_0.text(self.Column.VALUE).split(" ")[-1])
        old_channel_1_value = int(self.plot_tree_channel_1.text(self.Column.VALUE).split(" ")[-1])
        for count in range(5):
            QtTest.QTest.keyClick(self.plot.plot, QtCore.Qt.Key.Key_Right)
            self.processEvents(timeout=0.1)
            # Evaluate
            channel_0_value = int(self.plot_tree_channel_0.text(self.Column.VALUE).split(" ")[-1])
            channel_1_value = int(self.plot_tree_channel_1.text(self.Column.VALUE).split(" ")[-1])
            self.assertLess(old_channel_0_value, channel_0_value)
            self.assertGreater(old_channel_1_value, channel_1_value)
            old_channel_0_value = channel_0_value
            old_channel_1_value = channel_1_value

        # Press Key 'R' for range selection
        QtTest.QTest.keyClick(self.plot.plot, QtCore.Qt.Key.Key_R)
        self.processEvents(timeout=0.01)

        # Ensure that delta char is not present on channel values even if button is pressed
        self.assertNotIn("Δ", self.plot_tree_channel_0.text(self.Column.VALUE))
        self.assertNotIn("Δ", self.plot_tree_channel_1.text(self.Column.VALUE))

    def test_Plot_ChannelSelection_PushButton_ToggleBookmarks(self):
        """
        Precondition
            -

        Events:
            - Display 1 signal on plot
            - Press 3 times `Toggle bookmarks` button

        Evaluate:
            - Evaluate that bookmarks are not displayed before pressing `Toggle bookmarks` button
            - Evaluate that bookmarks are displayed after pressing `Toggle bookmarks` button first time
            - Evaluate that bookmarks are not displayed after pressing `Toggle bookmarks` button second time
        """
        # Hide channels
        iterator = QTreeWidgetItemIterator(self.plot.channel_selection)
        while iterator.value():
            item = iterator.value()
            if item.checkState(0) == QtCore.Qt.CheckState.Checked:
                item.setCheckState(0, QtCore.Qt.CheckState.Unchecked)
            iterator += 1

        # Press `Toggle bookmarks` button
        QtTest.QTest.mouseClick(self.plot.bookmark_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents(0.1)

        # Get expected colors
        pg_colors = Pixmap.color_names_exclude_defaults(self.plot.plot.viewport().grab())
        bookmarks_colors = COLORS[: len(COLORS) - len(self.plot.plot.bookmarks) - 1 : -1]

        # Evaluate
        self.assertTrue(self.plot.show_bookmarks)
        for color in bookmarks_colors:
            self.assertIn(color, pg_colors)
        self.assertIn(BLUE, pg_colors)

        # Press `Toggle bookmarks` button
        QtTest.QTest.mouseClick(self.plot.bookmark_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()
        # Evaluate
        self.assertFalse(self.plot.show_bookmarks)
        self.assertTrue(Pixmap.is_black(self.plot.plot.viewport().grab()))

    def test_Plot_ChannelSelection_PushButton_HideAxes(self):
        """
        Events:
            - Display 2 signal on plot
            - Press 3 times `Show axes` button
        Evaluate:
            - Evaluate that bookmarks are not displayed before pressing `Hide axes` button
            - Evaluate that bookmarks are displayed after pressing `Hide axes` button first time
            - Evaluate that bookmarks are not displayed after pressing `Hide axes` button second time
            _ Evaluate that only y_axis color are the same as selected channel color
        """
        # Hide channels
        iterator = QTreeWidgetItemIterator(self.plot.channel_selection)
        while iterator.value():
            item = iterator.value()
            if item.checkState(0) == QtCore.Qt.CheckState.Checked:
                item.setCheckState(0, QtCore.Qt.CheckState.Unchecked)
            iterator += 1

        if self.plot.plot.y_axis.isVisible() or self.plot.plot.x_axis.isVisible():
            QtTest.QTest.mouseClick(self.plot.hide_axes_btn, QtCore.Qt.MouseButton.LeftButton)
            # Evaluate
            self.assertFalse(self.plot.plot.y_axis.isVisible())
            self.assertFalse(self.plot.plot.x_axis.isVisible())

        # Press `Show axes` button
        QtTest.QTest.mouseClick(self.plot.hide_axes_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()

        # Evaluate
        self.assertTrue(self.plot.plot.y_axis.isVisible())
        self.assertTrue(self.plot.plot.x_axis.isVisible())

        with self.subTest("test pen color of axes"):
            self.assertNotEqual(self.plot_tree_channel_0.color.name(), self.plot_tree_channel_1.color.name())
            x_axis_color = self.plot.plot.x_axis.pen().color().name()

            # Event
            self.mouseClick_WidgetItem(self.plot_tree_channel_0)
            self.processEvents()

            # Evaluate
            self.assertEqual(x_axis_color, self.plot.plot.x_axis.pen().color().name())
            self.assertEqual(self.plot.plot.y_axis.pen().color().name(), self.plot_tree_channel_0.color.name())
            self.assertTrue(Pixmap.has_color(self.plot.plot.viewport().grab(), self.plot_tree_channel_0.color.name()))

            # Event
            self.mouseClick_WidgetItem(self.plot_tree_channel_1)
            self.processEvents()

            # Evaluate
            self.assertEqual(x_axis_color, self.plot.plot.x_axis.pen().color().name())
            self.assertEqual(self.plot.plot.y_axis.pen().color().name(), self.plot_tree_channel_1.color.name())
            self.assertTrue(Pixmap.has_color(self.plot.plot.viewport().grab(), self.plot_tree_channel_1.color.name()))

        # Press `Hide axes` button
        QtTest.QTest.mouseClick(self.plot.hide_axes_btn, QtCore.Qt.MouseButton.LeftButton)
        # Evaluate
        self.assertFalse(self.plot.plot.y_axis.isVisible())
        self.assertFalse(self.plot.plot.x_axis.isVisible())
        self.assertTrue(Pixmap.is_black(self.plot.plot.viewport().grab()))

    def test_Plot_ChannelSelection_PushButton_Lock(self):
        """
        Events:
            - Display 2 signal on plot
            - Press `Lock` button to lock y-axis
            - Move signal using drag-and-drop technique on both axis
            - Press `Lock` button to unlock y-axis
            - Move signal using drag-and-drop technique on both axis

        Evaluate:
            - Evaluate that if y-axis is locked, signal cannot be moved only on y-axis and common axis column is hidden
            - Evaluate that if y-axis is unlocked, signal can be moved on both x and y axes
              and common axis column isn't hidden
        """

        def move_signal(x, y):
            def drag_and_drop():
                center = self.plot.plot.viewport().geometry().center()
                QtTest.QTest.mouseMove(self.plot.plot.viewport(), QtCore.QPoint(center.x() + x, center.y() + y))

                QtTest.QTest.mouseRelease(
                    self.plot.plot.viewport(),
                    QtCore.Qt.MouseButton.LeftButton,
                    QtCore.Qt.KeyboardModifier.NoModifier,
                    QtCore.QPoint(center.x() + x, center.y() + y),
                    20,
                )

            self.mouseClick_WidgetItem(self.plot_tree_channel_0)
            QtCore.QTimer.singleShot(100, drag_and_drop)
            QtTest.QTest.mousePress(self.plot.plot.viewport(), QtCore.Qt.MouseButton.LeftButton)

        if self.plot.locked:
            QtTest.QTest.mouseClick(self.plot.lock_btn, QtCore.Qt.MouseButton.LeftButton)
            # Evaluate
            self.assertFalse(self.plot.locked)
            self.assertFalse(self.plot.channel_selection.isColumnHidden(self.plot.channel_selection.CommonAxisColumn))

        # Evaluate that signal wasn't moved on x-axis
        self.assertIsNone(self.plot_graph_channel_0.trim_info)

        # Press `Lock` button
        QtTest.QTest.mouseClick(self.plot.lock_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()
        sig_0_y_range = self.plot_graph_channel_0.y_range

        # move channels on plot graphics
        move_signal(50, 50)
        self.processEvents(0.5)

        # Evaluate
        self.assertTrue(self.plot.locked)
        self.assertTrue(self.plot.channel_selection.isColumnHidden(self.plot.channel_selection.CommonAxisColumn))
        self.assertTupleEqual(sig_0_y_range, self.plot_graph_channel_0.y_range)
        self.assertIsNotNone(self.plot_graph_channel_0.trim_info)

        # Press `Lock` button
        QtTest.QTest.mouseClick(self.plot.lock_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()

        # get trim info
        trim_info = self.plot_graph_channel_0.trim_info

        # move channels on plot graphics
        move_signal(-50, 50)
        self.processEvents(0.5)

        # Evaluate
        self.assertFalse(self.plot.locked)
        self.assertFalse(self.plot.channel_selection.isColumnHidden(self.plot.channel_selection.CommonAxisColumn))
        self.assertNotEqual(sig_0_y_range, self.plot_graph_channel_0.y_range)
        self.assertGreater(self.plot_graph_channel_0.trim_info[0], trim_info[0])  # start
        self.assertGreater(self.plot_graph_channel_0.trim_info[1], trim_info[1])  # stop
        self.assertEqual(self.plot_graph_channel_0.trim_info[2], trim_info[2])  # width

    def test_Plot_ChannelSelection_PushButtons_Zoom(self):
        """
        This method will test 4 buttons: zoom in, zoom out, undo zoom, redo zoom.
        Precondition:
            - Zoom history is clean - trim info must be None
            - Cursor is on center of plot. This step is required for easiest evaluation

        # Event
            - Press button `Zoom in`
            - Press button `Undo zoom`
            - Press button `Redo zoom`
            - Press button `Zoom out`
            - Press button `Zoom in`

        # Evaluate
            - Evaluate that `zoom` action was performed (zoom in will be tested at the end)
            - Evaluate that `undo zoom` action was performed
            - Evaluate that `redo zoom` action was performed and
             channels timestamp trim info is equal with its value before undo zoom action was performed
            - Evaluate that zoom `out action` was performed
            - Evaluate that zoom `in action` was performed

        """
        # Precondition
        self.assertIsNone(self.plot_graph_channel_0.trim_info)
        QtTest.QTest.mouseClick(self.plot.plot.viewport(), QtCore.Qt.MouseButton.LeftButton)

        buttons = self.plot.findChildren(QBtn)
        zoom_in_btn = "Zoom in"
        zoom_out_btn = "Zoom out"
        for btn in buttons:
            if btn.toolTip() == zoom_in_btn:
                zoom_in_btn = btn
                continue
            elif btn.toolTip() == zoom_out_btn:
                zoom_out_btn = btn
                continue

        # buttons was found
        self.assertIsInstance(zoom_in_btn, QBtn)
        self.assertIsInstance(zoom_out_btn, QBtn)

        # Event
        QtTest.QTest.mouseClick(zoom_in_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()
        trim_info_0 = self.plot_graph_channel_0.trim_info

        # Evaluate
        self.assertIsNotNone(trim_info_0)  # zoom was performed

        # Store previous zoom
        prev_zoom = trim_info_0[1] - trim_info_0[0]

        # Event
        QtTest.QTest.mouseClick(self.plot.undo_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()
        trim_info_1 = self.plot_graph_channel_0.trim_info

        # Evaluate
        self.assertGreater(trim_info_1[1] - trim_info_1[0], prev_zoom)

        # Store previous zoom
        prev_zoom = trim_info_1[1] - trim_info_1[0]

        # Event
        QtTest.QTest.mouseClick(self.plot.redo_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()
        trim_info_2 = self.plot_graph_channel_0.trim_info

        # Evaluate
        self.assertLess(trim_info_2[1] - trim_info_2[0], prev_zoom)
        self.assertTupleEqual(trim_info_0, trim_info_2)

        prev_zoom = trim_info_2[1] - trim_info_2[0]

        # Event
        QtTest.QTest.mouseClick(zoom_out_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()
        trim_info_3 = self.plot_graph_channel_0.trim_info

        # Evaluate
        self.assertGreater(trim_info_3[1] - trim_info_3[0], prev_zoom)

        # store previous zoom
        prev_zoom = trim_info_3[1] - trim_info_3[0]

        # Event
        QtTest.QTest.mouseClick(zoom_in_btn, QtCore.Qt.MouseButton.LeftButton)
        self.processEvents()
        trim_info_4 = self.plot_graph_channel_0.trim_info

        # Evaluate
        self.assertLess(trim_info_4[1] - trim_info_4[0], prev_zoom)
        self.assertTupleEqual(trim_info_2, trim_info_4)

    def test_Plot_ChannelSelection_PushButtons_CMD(self):
        """
        This method will test 6 actions from Cmd menu: Home, Honeywell, Fit, Stack, Increase font, Decrease font.

        Precondition:
            - No zoom actions performed. Y range for both signals are identical and timestamp trim info is None

        # Event
            -[0] Press `Cmd` button to open menu, after that select `Stack` action from menu
            -[1] Press `Cmd` button to open menu, after that select `Fit` action from menu
            -[2] Press `Cmd` button to open menu, after that select `Honeywell` action from menu
            -[3] Press `Cmd` button to open menu, after that select `Home` action from menu
            -[4] Press `Cmd` button to open menu, after that check common axis checkboxes
              and select `Stack` action from menu
            -[5] Press `Cmd` button to open menu, after that select `Increase font` action from menu
            -[6] Press `Cmd` button to open menu, after that select `Decrease font` action from menu

        # Evaluate
            -[0] Evaluate that signals Y ranges were modified,
              also Y range for first signal is less than Y range of second signal timestamp trim info is None
            -[1] Evaluate that signals Y ranges for both signals are identical, timestamp trim info is still None
            -[2] Evaluate that signals Y ranges for both signals are identical, timestamp trim info was modified,
              also trim info for both signals are identical
            -[3] Evaluate that signals Y ranges for both signals are identical, also timestamp trim info are identical,
              timestamp range became greater
            -[4] Evaluate that signals Y ranges for both signals are identical, also timestamp trim info are identical

            -[5] Evaluate that widget font size was increased
            -[6] Evaluate that widget font size was decreased
        """

        def click_on_cmd_action(btn: QBtn, action_text: str):
            for n, action in enumerate(btn.menu().actions(), 1):
                if action.text() == action_text:
                    break
            else:
                n = 0
            QtTest.QTest.mouseClick(btn, QtCore.Qt.MouseButton.LeftButton)
            for _ in range(n):
                QtTest.QTest.keyClick(btn.menu(), QtCore.Qt.Key.Key_Down)
                QtTest.QTest.qWait(100)
            QtTest.QTest.keyClick(btn.menu(), QtCore.Qt.Key.Key_Enter)

        # Precondition
        self.assertTupleEqual(self.plot_graph_channel_0.y_range, self.plot_graph_channel_1.y_range)
        self.assertIsNone(self.plot_graph_channel_0.trim_info)
        self.assertIsNone(self.plot_graph_channel_1.trim_info)

        buttons = self.plot.findChildren(QBtn)
        cmd_btn = "Cmd"
        for btn in buttons:
            if btn.text() == cmd_btn:
                cmd_btn = btn
                break

        self.processEvents(0.1)

        with self.subTest("PlotGraphics tests"):
            # Event
            click_on_cmd_action(cmd_btn, "Stack")
            self.processEvents(0.1)

            # Evaluate
            self.assertLess(self.plot_graph_channel_0.y_range[0], self.plot_graph_channel_1.y_range[0])
            self.assertLess(self.plot_graph_channel_0.y_range[1], self.plot_graph_channel_1.y_range[1])

            self.assertIsNone(self.plot_graph_channel_0.trim_info)  # zoom on timestamp axis wasn't performed
            self.assertIsNone(self.plot_graph_channel_1.trim_info)

            # Event
            click_on_cmd_action(cmd_btn, "Fit")
            self.processEvents(0.1)

            # Evaluate
            self.assertTupleEqual(self.plot_graph_channel_0.y_range, self.plot_graph_channel_1.y_range)
            self.assertIsNone(self.plot_graph_channel_0.trim_info)
            self.assertIsNone(self.plot_graph_channel_1.trim_info)

            # Event
            click_on_cmd_action(cmd_btn, "Honeywell")
            self.processEvents(0.1)

            # Evaluate
            self.assertEqual(self.plot_graph_channel_0.y_range[0], self.plot_graph_channel_1.y_range[0])
            self.assertEqual(self.plot_graph_channel_0.y_range[1], self.plot_graph_channel_1.y_range[1])
            self.assertTupleEqual(self.plot_graph_channel_0.trim_info, self.plot_graph_channel_1.trim_info)

            # save previous trim info
            prev_trim_info = self.plot_graph_channel_0.trim_info[1] - self.plot_graph_channel_0.trim_info[0]
            # Event
            click_on_cmd_action(cmd_btn, "Home")
            self.processEvents(0.1)

            # Evaluate
            self.assertEqual(self.plot_graph_channel_0.y_range[0], self.plot_graph_channel_1.y_range[0])
            self.assertEqual(self.plot_graph_channel_0.y_range[1], self.plot_graph_channel_1.y_range[1])
            self.assertLess(
                prev_trim_info, self.plot_graph_channel_0.trim_info[1] - self.plot_graph_channel_0.trim_info[0]
            )
            self.assertTupleEqual(self.plot_graph_channel_0.trim_info, self.plot_graph_channel_1.trim_info)

            # Event
            # Set checked individual axis for both channels
            common_axis_column = self.plot_tree_channel_0.CommonAxisColumn
            if self.plot_tree_channel_0.checkState(common_axis_column) == QtCore.Qt.CheckState.Unchecked:
                self.plot_tree_channel_0.setCheckState(common_axis_column, QtCore.Qt.CheckState.Checked)
            if self.plot_tree_channel_1.checkState(common_axis_column) == QtCore.Qt.CheckState.Unchecked:
                self.plot_tree_channel_1.setCheckState(common_axis_column, QtCore.Qt.CheckState.Checked)

            click_on_cmd_action(cmd_btn, "Stack")
            self.processEvents(0.1)

            # Evaluate
            self.assertTupleEqual(self.plot_graph_channel_0.y_range, self.plot_graph_channel_1.y_range)
            self.assertTupleEqual(self.plot_graph_channel_0.trim_info, self.plot_graph_channel_1.trim_info)

        with self.subTest("text actions tests"):
            prev_text_size = self.plot.font().pointSize()

            # Event
            click_on_cmd_action(cmd_btn, "Increase font")
            self.processEvents(0.1)

            # Evaluate
            self.assertGreater(self.plot.font().pointSize(), prev_text_size)

            prev_text_size = self.plot.font().pointSize()

            # Event
            click_on_cmd_action(cmd_btn, "Decrease font")
            self.processEvents(0.1)

            # Evaluate
            self.assertLess(self.plot.font().pointSize(), prev_text_size)



#!/usr/bin/env python
from pathlib import Path
from unittest import mock

from PySide6.QtGui import QKeySequence
from PySide6.QtTest import QTest
from PySide6.QtWidgets import QApplication, QTreeWidgetItemIterator

from asammdf.gui.widgets import numeric, plot, tabular
from asammdf.gui.widgets.file import FileWidget
from asammdf.gui.widgets.main import MainWindow
from test.asammdf.gui.test_base import TestBase
from test.asammdf.gui.widgets.test_BaseFileWidget import TestFileWidget


class TestShortcuts(TestBase):
    def setUp(self):
        """
        Events:
            Open Main window
        Returns
        -------

        """
        super().setUp()
        self.measurement_file = str(Path(TestBase.resource, "ASAP2_Demo_V171.mf4"))
        self.mw = MainWindow(files=(self.measurement_file,))
        self.mw.showNormal()
        self.processEvents(1)

        # get shortcuts
        self.shortcuts = TestFileWidget.load_shortcuts_from_json_file(self, self.mw)
        self.assertIsNotNone(self.shortcuts)

    def destroyMW(self):
        if self.mw:
            self.mw.close()
            self.mw.deleteLater()

    def tearDown(self):
        if self.mw:
            self.destroyMW()

    def test_fullscreen_shortcut(self):
        """
        Test scope:
            Ensure that F11 shortcut will toggle full-screen mode for opened file widget,
                and after closing the main window, file widget will be destroyed.
        Events:
            - Open valid measurement file
            - Press key `F11`
            - Close MainWindow

        Evaluate:
            - Evaluate that file widget isn't in full-screen mode by default.
            - Evaluate that after pressing key F11, file widget is in full-screen mode
            - Evaluate that after closing the main window, there are no more opened widgets.

        :return:
        """
        # Setup
        self.processEvents(0.01)
        # file widget
        file_widget = self.mw.files.widget(0)

        # Evaluate
        self.assertFalse(file_widget.isFullScreen())

        # Event
        QTest.keySequence(self.mw, QKeySequence(self.shortcuts["toggle_full-screen"]))  # Press F11
        self.processEvents()

        # Evaluate
        self.assertTrue(file_widget.isFullScreen())

        # Close MainWindow
        self.destroyMW()
        self.processEvents()

        # Evaluate
        for w in QApplication.topLevelWidgets():
            if isinstance(w, FileWidget):
                self.assertFalse(w.isVisible())

    def test_create_plot__numeric__tabular_sub_window_shortcut(self):
        """
        Test scope:
            Ensure that Plot, Numeric and Tabular sub-windows can be created by pressing keys F2, F3 and F4

        Events:
            - Open valid file
            - Press key F2.
            - Press key F3.
            - Press key F4.
            - Search all "*matrix*" channels and set all of them checked
            - Press key F2.
            - Press key F3.
            - Press key F4.

        Evaluate:
            - Evaluate that after pressing key F2, there is one new Plot type sub-window.
            - Evaluate that after pressing key F3, there is one new Numeric type sub-window.
            - Evaluate that after pressing key F4, there is one new Tabular type sub-window.
            - Evaluate that after pressing key F2, there is one new Plot type sub-window with checked channels.
            - Evaluate that after pressing key F3, there is one new Numeric type sub-window with checked channels.
            - Evaluate that after pressing key F4, there is one new Tabular type sub-window with checked channels.
        :return:
        """
        # Setup
        windows_count = 0

        # file widget
        file_widget = self.mw.files.widget(0)

        with self.subTest("test_shortcut_F2_without_selected_channels"):
            # Event
            QTest.keySequence(self.mw, QKeySequence(self.shortcuts["create_plot_window"]))  # Press F2
            self.processEvents()

            # Evaluate
            self.assertEqual(len(file_widget.mdi_area.subWindowList()), windows_count + 1)
            self.assertTrue(isinstance(file_widget.mdi_area.subWindowList()[windows_count].widget(), plot.Plot))

            windows_count += 1

        with self.subTest("test_shortcut_F3_without_selected_channels"):
            # Event
            QTest.keySequence(self.mw, QKeySequence(self.shortcuts["create_numeric_window"]))  # Press F3
            self.processEvents()

            # Evaluate
            self.assertEqual(len(file_widget.mdi_area.subWindowList()), windows_count + 1)
            self.assertTrue(isinstance(file_widget.mdi_area.subWindowList()[windows_count].widget(), numeric.Numeric))

            windows_count += 1

        with self.subTest("test_shortcut_F4_without_selected_channels"):
            # Event
            QTest.keySequence(self.mw, QKeySequence(self.shortcuts["create_tabular_window"]))  # Press F4
            self.processEvents()

            # Evaluate
            self.assertEqual(len(file_widget.mdi_area.subWindowList()), windows_count + 1)
            self.assertTrue(isinstance(file_widget.mdi_area.subWindowList()[windows_count].widget(), tabular.Tabular))

            windows_count += 1

        # Setup
        matrix_items = {}
        pattern = "matrix"
        # Search signals with specific patter
        iterator = QTreeWidgetItemIterator(file_widget.channels_tree)
        while item := iterator.value():
            if pattern.upper() in item.name.upper():
                matrix_items[item.entry] = item.name
            iterator += 1
        sw_count = 0  # Sub-windows
        # Mock for Advanced search and windowSelectionDialog objects
        with mock.patch("asammdf.gui.widgets.file.AdvancedSearch") as mo_AdvancedSearch:
            mo_AdvancedSearch.return_value.result = matrix_items
            mo_AdvancedSearch.return_value.pattern_window = False
            mo_AdvancedSearch.return_value.add_window_request = False
            # Check some channels
            QTest.keySequence(self.mw, QKeySequence("Ctrl+F"))  # Press Ctrl+F

            self.processEvents(0.01)
        # Evaluate
        mo_AdvancedSearch.assert_called()

        with self.subTest("test_shortcut_F2_with_selected_channels"):
            # Event
            QTest.keySequence(self.mw, QKeySequence(self.shortcuts["create_plot_window"]))  # Press F2
            self.processEvents()

            # Evaluate
            self.assertEqual(len(file_widget.mdi_area.subWindowList()), windows_count + 1)
            self.assertTrue(isinstance(file_widget.mdi_area.subWindowList()[windows_count].widget(), plot.Plot))

            cs = file_widget.mdi_area.subWindowList()[windows_count].widget().channel_selection

            # Evaluate plot widget
            self.assertEqual(len(matrix_items), cs.topLevelItemCount())
            iterator = QTreeWidgetItemIterator(cs)
            while item := iterator.value():
                self.assertIn(item.name, matrix_items.values())
                iterator += 1
            windows_count += 1

        with self.subTest("test_shortcut_F3_with_selected_channels"):
            # Event
            QTest.keySequence(self.mw, QKeySequence(self.shortcuts["create_numeric_window"]))  # Press F3
            self.processEvents()

            # Evaluate
            self.assertEqual(len(file_widget.mdi_area.subWindowList()), windows_count + 1)
            self.assertTrue(isinstance(file_widget.mdi_area.subWindowList()[windows_count].widget(), numeric.Numeric))

            signals = file_widget.mdi_area.subWindowList()[windows_count].widget().channels.backend.signals
            self.assertEqual(len(matrix_items), len(signals))
            for sig in signals:
                self.assertIn(sig.name, matrix_items.values())
            windows_count += 1

        with self.subTest("test_shortcut_F4_with_selected_channels"):
            # Event
            QTest.keySequence(self.mw, QKeySequence(self.shortcuts["create_tabular_window"]))  # Press F4
            self.processEvents()

            # Evaluate
            self.assertEqual(len(file_widget.mdi_area.subWindowList()), windows_count + 1)
            self.assertTrue(isinstance(file_widget.mdi_area.subWindowList()[windows_count].widget(), tabular.Tabular))

            signals = file_widget.mdi_area.subWindowList()[windows_count].widget().tree
            # channels columns count - timestamp column
            self.assertEqual(len(matrix_items), signals.columnHeader.model().columnCount() - 1)
            for key in matrix_items.values():
                self.assertIn(key, signals.pgdf.df.keys())



from copy import deepcopy
import os

from PySide6 import QtCore, QtWidgets

from ..dialogs.messagebox import MessageBox
from ..widgets.functions_manager import FunctionsManager


class FunctionsManagerDialog(QtWidgets.QDialog):
    def __init__(
        self,
        definitions,
        channels=None,
        selected_definition="",
        prefix="",
        global_variables="",
        *args,
        **kwargs,
    ):
        definitions = definitions or {}
        definitions = {
            name: {
                "definition": definition,
                "uuid": os.urandom(6).hex(),
            }
            for name, definition in definitions.items()
        }

        self.original_definitions = {}
        self.modified_definitions = {}
        self.global_variables = ""

        for name, info in definitions.items():
            self.original_definitions[info["uuid"]] = {
                "name": name,
                "definition": info["definition"],
            }

        super().__init__(*args, **kwargs)

        self.setObjectName("FunctionsManagerDialog")
        self.resize(404, 294)
        self.setSizeGripEnabled(True)
        self.setWindowFlags(QtCore.Qt.WindowType.Window)
        self.verticalLayout = QtWidgets.QVBoxLayout(self)
        self.widget = FunctionsManager(deepcopy(definitions), channels, selected_definition, global_variables)

        self.verticalLayout.addWidget(self.widget)

        self.horLayout = QtWidgets.QHBoxLayout(self)

        spacer = QtWidgets.QSpacerItem(
            40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum
        )
        self.apply_btn = QtWidgets.QPushButton("Apply")
        self.cancel_btn = QtWidgets.QPushButton("Cancel")
        self.horLayout.addSpacerItem(spacer)
        self.horLayout.addWidget(self.apply_btn)
        self.horLayout.addWidget(self.cancel_btn)

        self.verticalLayout.addLayout(self.horLayout)

        self.apply_btn.clicked.connect(self.apply)
        self.cancel_btn.clicked.connect(self.cancel)
        self.pressed_button = "cancel"

        if prefix:
            self.setWindowTitle(f"{prefix} - Functions Manager")
        else:
            self.setWindowTitle("Functions Manager")

    def apply(self, *args):
        self.pressed_button = "apply"
        self.modified_definitions = {}

        self.widget.refresh_definitions()

        for name, info in self.widget.definitions.items():
            if info["current_definition"] != info["definition"]:
                result = MessageBox.question(
                    self, "Unsaved function definitions", "Do you want to review the functions before exporting them?"
                )
                if result == MessageBox.StandardButton.No:
                    break
                else:
                    for row in range(self.widget.functions_list.count()):
                        item = self.widget.functions_list.item(row)
                        if item.text() == name:
                            self.widget.functions_list.setCurrentRow(row)
                            self.widget.functions_list.scrollToItem(item)
                            break
                    return

        for name, info in self.widget.definitions.items():
            self.modified_definitions[info["uuid"]] = {
                "name": name,
                "definition": info["current_definition"],
            }

        self.global_variables = "\n".join(
            line for line in self.widget.globals_definition.toPlainText().splitlines() if line
        )

        self.close()

    def cancel(self, *args):
        self.pressed_button = "cancel"
        self.close()
